@@ -1,8 +1,8 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 1994, 1999, 2007-2012 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -18,6 +18,9 @@
  with this program; if not, write to the Free Software Foundation, Inc.,
  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
+
+#include "ranges.h"
+
 extern int slrn_select_group_mode (void);
 extern void slrn_group_quit (void);
 extern int slrn_add_group (char *);
@@ -28,7 +31,7 @@ extern void slrn_list_all_groups (int);
 extern void slrn_hide_groups (int);
 extern int slrn_read_newsrc (int);
 extern int slrn_write_newsrc (int);
-extern void slrn_mark_article_as_read (char *, long);
+extern void slrn_mark_articles_as_read (char *, NNTP_Artnum_Type, NNTP_Artnum_Type);
 extern int slrn_get_new_groups (int);
 extern void slrn_init_group_mode (void);
 extern void slrn_get_group_descriptions (void);
@@ -46,41 +49,38 @@ extern void slrn_hide_current_group (void);
 extern void slrn_intr_get_group_order (void);
 extern void slrn_intr_set_group_order (void);
 
-typedef struct Slrn_Range_Type 
-{
-   struct Slrn_Range_Type *next;
-   struct Slrn_Range_Type *prev;
-   int min, max;
-} Slrn_Range_Type;
-
 typedef struct Slrn_Group_Type
 {
    struct Slrn_Group_Type *next;
    struct Slrn_Group_Type *prev;
    unsigned int flags;
-#define GROUP_UNSUBSCRIBED		0x001
-#define GROUP_NEW_GROUP_FLAG		0x002
-#define GROUP_HARMLESS_FLAGS_MASK 	0x0FF
-#define GROUP_HIDDEN			0x100
-#define GROUP_TOUCHED			0x200
-#define GROUP_PROCESSED			0x400
+#define GROUP_UNSUBSCRIBED		0x0001
+#define GROUP_NEW_GROUP_FLAG		0x0002
+#define GROUP_HARMLESS_FLAGS_MASK 	0x00FF
+#define GROUP_HIDDEN			0x0100
+#define GROUP_TOUCHED			0x0200
+#define GROUP_PROCESSED			0x0400
 
    char *group_name;
    unsigned long hash;
    struct Slrn_Group_Type *hash_next;
-   
+
    Slrn_Range_Type range;		       /* the first range corresponds to
 						* what the server has.  next ranges
 						* correspond to what has been read.
 						*/
-   int unread;
-   char *descript;		       /* description of the group */
+   Slrn_Range_Type *requests; /* body requests for true offline mode
+			       * note that the first one is also malloced here! */
+   int requests_loaded;	/* We load these each time we enter article mode in case
+			 * slrnpull ran in the background */
+
+   NNTP_Artnum_Type unread;
+   char *descript;		       /* description of the group -- malloced, could be NULL */
 }
 Slrn_Group_Type;
 
 extern Slrn_Group_Type *Slrn_Group_Current_Group;
-/* See important comment in group.c about this. */
-extern void slrn_add_group_ranges (Slrn_Group_Type *, int, int);
+extern void slrn_group_recount_unread (Slrn_Group_Type *);
 
 extern char *Slrn_Group_Help_Line;
 extern char *Slrn_Group_Status_Line;
@@ -96,3 +96,4 @@ extern int Slrn_Query_Group_Cutoff;
 extern int Slrn_Write_Newsrc_Flags;
 extern int Slrn_Display_Cursor_Bar;
 extern int Slrn_Drop_Bogus_Groups;
+extern int Slrn_Max_Queued_Groups;
