@@ -1,7 +1,9 @@
+/* -*- mode: C; mode: fold; -*- */
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 1994, 1999, 2007-2012 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -30,11 +32,14 @@
 #include <slang.h>
 #include "jdmacros.h"
 
+#include "help.h"
 #include "slrn.h"
+#include "snprintf.h"
 #include "misc.h"
 #include "util.h"
+#include "strutil.h"
 
-static char *Global_Help [] = 
+static char *Global_Help [] =
 {
    "",
    "",
@@ -43,9 +48,8 @@ static char *Global_Help [] =
    N_(" Questions not covered by the documentation can be posted to"),
    N_("   news.software.readers"),
    "",
-   N_(" Please email bug reports, suggestions or comments to the current maintainer,"),
-   N_("   Thomas Schultz <tststs@gmx.de>"),
-   N_(" The original author of this program is John E. Davis <davis@space.mit.edu>"),
+   N_(" Please email bug reports, suggestions or comments to the author of this"),
+   N_(" program, John E. Davis <jed@jedsoft.org>"),
    NULL
 };
 
@@ -122,12 +126,10 @@ static char *Art_Help[] =
    N_("  \\                  Show signature (on/off)."),
    N_("  W                  Wrap long lines (on/off)."),
 #if SLRN_HAS_SPOILERS
-   N_("  ESC ?              Reveal spoilers (on/off)."),
+   N_("  ESC ?              Reveal spoilers."),
 #endif
    N_("  ]                  Show PGP signature (on/off)."),
-#if SLRN_HAS_VERBATIM_MARKS
    N_("  [                  Show verbatim marks (on/off)."),
-#endif
    N_("  Ctrl-^             Enlarge the article window."),
    N_("  ^                  Shrink the article window."),
    N_("  z                  Maximize / Unmaximize the article window."),
@@ -141,6 +143,9 @@ static char *Art_Help[] =
    N_("Miscellaneous actions:"),
    N_("  K                  Create a scorefile entry interactively."),
    N_("      ESC 1 K        Edit scorefile."),
+#if SLRN_HAS_SPOOL_SUPPORT
+   N_("  m                  (Un-)mark article body for download by slrnpull."),
+#endif
    N_("  v                  Show which scorefile rules matched the current article."),
    N_("  * The following five commands query the server if necessary:"),
    N_("  ESC l              Locate article by its Message-ID."),
@@ -174,8 +179,8 @@ static char *Group_Help [] =
    N_("  UP                      Go to the previous group."),
    N_("  Ctrl-V, Ctrl-D          Scroll to the next page."),
    N_("  ESC V, Ctrl-U           Scroll to the previous page."),
-   N_("  ESC <                   Go to the bottom of the list."),
-   N_("  ESC >                   Go to the top of the list."),
+   N_("  ESC >                   Go to the bottom of the list."),
+   N_("  ESC <                   Go to the top of the list."),
    N_("  /                       Group keyword search."),
    N_("Actions:"),
    N_("  SPACE, RETURN           Enter the current newsgroup."),
@@ -217,10 +222,10 @@ static char *Group_Help [] =
 
 static char *Copyright_Notice [] =
 {
-   " Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>",
+   " Copyright (c) 1994, 1999, 2007-2012 John E. Davis <jed@jedsoft.org>",
      "",
      " For parts of it:",
-     " Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>",
+     " Copyright (c) 2001-2003 Thomas Schultz <tststs@gmx.de>",
      "",
      " For the parts in src/snprintf.c that are based on code from glib 1.2.8:",
      " Copyright (C) 1995-1998  Peter Mattis, Spencer Kimball and Josh MacDonald",
@@ -276,20 +281,20 @@ static void do_help (char **help)
    char **p, *sect = NULL;
    char quit;
    char **this_help;
-   
+
    this_help = p = help;
-   
+
    slrn_enable_mouse (0);
-   
+
    while (1)
      {
 	i = 0;
 	if (*p == NULL) break;
-	
+
 	slrn_push_suspension (0);
 
 	SLsmg_cls ();
-	
+
 	if ((sect != NULL) && (**p == ' '))
 	  {
 	     SLsmg_set_color (1);
@@ -315,11 +320,11 @@ The following copyright notice applies to the slrn newsreader:"));
 	       }
 	     i++;
 	  }
-	
+
 	while (i < SLtt_Screen_Rows - 4)
 	  {
 	     char pp;
-	     
+
 	     if (*p == NULL)
 	       {
 		  if ((this_help == Global_Help) ||
@@ -327,7 +332,7 @@ The following copyright notice applies to the slrn newsreader:"));
 		  this_help = p = Global_Help;
 		  sect = NULL;
 	       }
-	     
+
 	     pp = **p;
 	     if ((pp != ' ') && pp)
 	       {
@@ -336,7 +341,7 @@ The following copyright notice applies to the slrn newsreader:"));
 		  i++;
 		  SLsmg_set_color (1);
 	       }
-	     
+
 	     SLsmg_gotorc (i, 0);
 	     if (**p != '\0')
 	       SLsmg_write_string (_(*p));
@@ -348,17 +353,17 @@ The following copyright notice applies to the slrn newsreader:"));
 	       }
 	     p++;
 	  }
-	
+
 	SLsmg_gotorc (i + 1, 0);
 	SLsmg_set_color (1);
-	
+
 	if ((*p == NULL)
 	    && (this_help == help))
 	  {
 	     this_help = p = Global_Help;
 	     sect = NULL;
 	  }
-	
+
 	if (*p == NULL)
 	  {
 	     if (this_help == Copyright_Notice)
@@ -377,9 +382,9 @@ Press 'q' to quit help, 'c' to start over, or any other key to continue."));
 	       SLsmg_write_string (_("\
 Press 'q' to quit help, '?' to start over, or any other key to continue."));
 	  }
-	
+
 	slrn_smg_refresh ();
-	
+
 	slrn_pop_suspension ();
 
 	SLang_flush_input ();
@@ -413,22 +418,25 @@ int slrn_parse_helpfile (char *helpfile)
    char **current_help = NULL;
    int num_lines = 0;
    unsigned char *b;
+   int status;
 
    if (Slrn_Batch)
      return 0;
-   
+
    if (NULL == (fp = fopen (helpfile, "r"))) return -1;
-   while (fgets (buf, sizeof (buf) - 1, fp) != NULL)
+
+   status = -1;
+   while (NULL != fgets (buf, sizeof (buf) - 1, fp))
      {
 	ch = *buf;
-	
+
 	/* Skip over common comments */
 	if ((ch == '#') || (ch == '%') || (ch == ';') || (ch == '!'))
 	  continue;
-	
+
 	b = (unsigned char *) slrn_skip_whitespace (buf);
 	if (*b == 0) continue;
-	
+
 	if (ch == '[')
 	  {
 	     /* end current help */
@@ -437,7 +445,7 @@ int slrn_parse_helpfile (char *helpfile)
 		  slrn_free (current_help[num_lines]);
 		  current_help[num_lines] = NULL;
 	       }
-	     
+
 	     num_lines = 0;
 	     ch = *(buf + 1) | 0x20;
 	     if (ch == 'a')
@@ -446,38 +454,44 @@ int slrn_parse_helpfile (char *helpfile)
 	       }
 	     else if (ch == 'g') current_help = User_Group_Help;
 	     else current_help = NULL;
-	     
+
 	     continue;
 	  }
-	
+
 	if (current_help == NULL) continue;
-	
+
 	if (MAX_HELP_LINES == num_lines + 1)
 	  {
 	     current_help[num_lines] = NULL;
 	     current_help = NULL;
 	     continue;
 	  }
-	
+
 	slrn_free (current_help [num_lines]);
-	
-	if (NULL != (current_help [num_lines] = (char *) slrn_strmalloc (buf, 0)))
-	  num_lines++;
+
+	if (NULL == (current_help [num_lines] = (char *) slrn_strmalloc (buf, 0)))
+	  goto return_error;
+
+	num_lines++;
      }
+
+   status = 0;
+   /* drop */
+
+return_error:
    if (current_help != NULL)
      {
 	slrn_free (current_help[num_lines]);
 	current_help[num_lines] = NULL;
      }
    slrn_fclose (fp);
-   return 0;
+   return status;
 }
 
-
 void slrn_article_help (void)
 {
    char **h;
- 
+
    if (Slrn_Batch) return;
    if (User_Article_Help[0] != NULL) h = User_Article_Help; else h = Art_Help;
    do_help (h);
@@ -491,3 +505,376 @@ void slrn_group_help (void)
    if (User_Group_Help[0] != NULL) h = User_Group_Help; else h = Group_Help;
    do_help (h);
 }
+
+/* Returns the key sequence to which function f in the given keymap is
+ * bound (NULL if unbound). If more than one key sequences apply, a random
+ * one gets returned.
+ * The returned string can be static and does not need to be freed. */
+char *slrn_help_keyseq_from_function (char *f, SLKeyMap_List_Type *map) /*{{{*/
+{
+   int i;
+   SLang_Key_Type *key, *key_root;
+   FVOID_STAR fp;
+   unsigned char type;
+   static char buf[3];
+
+   if (NULL == (fp = (FVOID_STAR) SLang_find_key_function(f, map)))
+     type = SLKEY_F_INTERPRET;
+   else type = SLKEY_F_INTRINSIC;
+
+   i = 256;
+   key_root = map->keymap;
+   while (i--)
+     {
+	key = key_root->next;
+	if ((key == NULL) && (type == key_root->type) &&
+	    (((type == SLKEY_F_INTERPRET) && (!strcmp((char *) f, key_root->f.s)))
+	     || ((type == SLKEY_F_INTRINSIC) && (fp == key_root->f.f))))
+	  {
+	     buf[0] = 2;
+	     buf[1] = 256 - 1 - i;
+	     buf[2] = 0;
+	     return buf;
+	  }
+
+	while (key != NULL)
+	  {
+	     if ((key->type == type) &&
+		 (((type == SLKEY_F_INTERPRET) && (!strcmp((char *) f, key->f.s)))
+		  || ((type == SLKEY_F_INTRINSIC) && (fp == key->f.f))))
+	       {
+		  return (char*)key->str;
+	       }
+	     key = key->next;
+	  }
+	key_root++;
+     }
+   return NULL;
+}
+/*}}}*/
+
+/* The following code handles conversion between (escape) key sequences
+ * and their symbolic names, like "<Up>" */
+
+/* In the following arrays, the same index corresponds to the same key */
+
+#define NUMBER_OF_KEYNAMES 37
+
+/* Symbolic names of keys as shown to the user */
+static char *KeyNames[NUMBER_OF_KEYNAMES] = /*{{{*/
+{
+   N_( "<PageUp>" ),
+   N_( "<PageDown>" ),
+   N_( "<Up>" ),
+   N_( "<Down>" ),
+   N_( "<Right>" ),
+   N_( "<Left>" ),
+   N_( "<Delete>" ),
+   N_( "<BackSpace>" ),
+   N_( "<Insert>" ),
+   N_( "<Home>" ),
+   N_( "<End>" ),
+   N_( "<Enter>" ),
+   N_( "<Return>" ),
+   N_( "<Tab>" ),
+   N_( "<BackTab>" ),
+   N_( "<Space>" ),
+   N_( "<F1>" ),
+   N_( "<F2>" ),
+   N_( "<F3>" ),
+   N_( "<F4>" ),
+   N_( "<F5>" ),
+   N_( "<F6>" ),
+   N_( "<F7>" ),
+   N_( "<F8>" ),
+   N_( "<F9>" ),
+   N_( "<F10>" ),
+   N_( "<F11>" ),
+   N_( "<F12>" ),
+   N_( "<F13>" ),
+   N_( "<F14>" ),
+   N_( "<F15>" ),
+   N_( "<F16>" ),
+   N_( "<F17>" ),
+   N_( "<F18>" ),
+   N_( "<F19>" ),
+   N_( "<F20>" ),
+   N_( "<Esc>" )
+}; /*}}}*/
+
+/* Length information for the unlocalized versions */
+static unsigned char KeyNameLengths[NUMBER_OF_KEYNAMES] =
+{8,10,4,6,7,6,8,11,8,6,5,7,8,5,9,7,4,4,4,4,4,4,4,4,4,5,5,5,5,5,5,5,5,5,5,5,5};
+
+#ifdef REAL_UNIX_SYSTEM
+/* Symbolic names of keys in the termcap database */
+static char *TermcapNames[NUMBER_OF_KEYNAMES] = /*{{{*/
+{
+   "kP",
+     "kN",
+     "ku",
+     "kd",
+     "kr",
+     "kl",
+     "kD",
+     "kb",
+     "kI",
+     "kh",
+     "@7",
+     "@8",
+     "",
+     "",
+     "kB",
+     "",
+     "k1",
+     "k2",
+     "k3",
+     "k4",
+     "k5",
+     "k6",
+     "k7",
+     "k8",
+     "k9",
+     "k;",
+     "F1",
+     "F2",
+     "F3",
+     "F4",
+     "F5",
+     "F6",
+     "F7",
+     "F8",
+     "F9",
+     "FA",
+     ""
+}; /*}}}*/
+#endif
+
+/* Default key sequences as fallbacks when termcap lookup has no result */
+static char *DefaultSequences[NUMBER_OF_KEYNAMES] = /*{{{*/
+{
+#ifdef IBMPC_SYSTEM
+   "\xE0I",
+     "\xE0Q",
+     "\xE0H",
+     "\xE0P",
+     "\xE0M",
+     "\xE0K",
+     "\xE0S",
+     "^@S",
+     "\xE0R",
+     "\xE0G",
+     "\xE0O",
+#else /* NOT IBMPC_SYSTEM */
+     "\033[5~",
+     "\033[6~",
+     "\033[A",
+     "\033[B",
+     "\033[C",
+     "\033[D",
+     "\033[3~",
+     "\010",
+     "\033[2~",
+     "\033[1~",
+     "\033[4~",
+#endif /* NOT IBMPC_SYSTEM */
+     "\r",
+     "\r",
+     "\t",
+     "\033[Z",
+     " ",
+#ifdef IBMPC_SYSTEM
+     "^@;",
+     "^@<",
+     "^@=",
+     "^@>",
+     "^@?",
+     "^@@",
+     "^@A",
+     "^@B",
+     "^@C",
+     "^@D",
+     "^@\x85",
+     "^@\x86",
+     "^@T",
+     "^@U",
+     "^@V",
+     "^@W",
+     "^@X",
+     "^@Y",
+     "^@Z",
+     "^@[",
+#else /* NOT IBMPC_SYSTEM */
+     "\033[11~",
+     "\033[12~",
+     "\033[13~",
+     "\033[14~",
+     "\033[15~",
+     "\033[17~",
+     "\033[18~",
+     "\033[19~",
+     "\033[20~",
+     "\033[21~",
+     "\033[23~",
+     "\033[24~",
+     "\033[25~",
+     "\033[26~",
+     "\033[28~",
+     "\033[29~",
+     "\033[31~",
+     "\033[32~",
+     "\033[33~",
+     "\033[34~",
+#endif /* NOT IBMPC_SYSTEM */
+     "\033"
+}; /*}}}*/
+
+/* Key sequences we actually use, initialized by init_keysym_table */
+/* The first character denotes the length of the sequence.
+ * Additionally, the strings are nul-terminated. */
+static char EscapeSequences[NUMBER_OF_KEYNAMES][SLANG_MAX_KEYMAP_KEY_SEQ+2];
+
+/* This function has to be called once before the conversion function can be
+ * used. */
+void slrn_help_init_keysym_table (void) /*{{{*/
+{
+   int i;
+   for (i = 0; i<NUMBER_OF_KEYNAMES; i++)
+     {
+#ifdef REAL_UNIX_SYSTEM
+	if (*TermcapNames[i] != 0) /* try to find in termcap database */
+	  {
+	     char *s = SLtt_tgetstr (TermcapNames[i]);
+	     int len;
+	     if (s != NULL)
+	       {
+		  len = strlen(s);
+		  if (len<=SLANG_MAX_KEYMAP_KEY_SEQ)
+		    {
+		       strcpy(EscapeSequences[i]+1,s); /* safe */
+		       EscapeSequences[i][0] = len;
+		       continue;
+		    }
+	       }
+	  }
+#endif
+	/* Fall back to the compiled-in default */
+	strcpy(EscapeSequences[i]+1,DefaultSequences[i]); /* safe */
+	EscapeSequences[i][0] = strlen(DefaultSequences[i]);
+     }
+}
+/*}}}*/
+
+/* Returns a human-friendly string representation of the given key sequence.
+ * Please note that it is stored in a local static variable.
+ * May return NULL if the resulting string would be too large. */
+char *slrn_help_keyseq_to_string (char *key, int keylen) /*{{{*/
+{
+   const int maxlen = 30;
+   static char result[31]; /* maxlen+1 */
+   int ind = 0;
+
+   while (keylen && (ind < maxlen))
+     {
+	int i;
+	for (i = 0; i < NUMBER_OF_KEYNAMES; i++)
+	  {
+/* On IBMPC_SYSTEM, "escape" sequences may begin with the nul character, which
+ * requires special case treatment. Sigh.
+ * We replaced nul with "^@" above so we can still use the str* functions */
+	     if ((EscapeSequences[i][0]<=keylen) &&
+		 (((*key == 0) && /* this part for IBMPC_SYSTEM */
+		   (EscapeSequences[i][0]>=2) &&
+		   (EscapeSequences[i][1] == '^') &&
+		   (EscapeSequences[i][2] == '@') &&
+		   !strncmp(EscapeSequences[i]+3,key+1, EscapeSequences[i][0]-2)) ||
+		  !strncmp(EscapeSequences[i]+1, key, EscapeSequences[i][0])))
+	       break;
+	  }
+	if (i == NUMBER_OF_KEYNAMES) /* no key name found */
+	  {
+	     if (*key < 32) /* non-printable */
+	       {
+		  char *ctrlstr = _("Ctrl-");
+		  int len = strlen(ctrlstr)+1;
+		  if (len > maxlen-ind)
+		    break; /* not enough memory */
+		  strcpy (result+ind, ctrlstr); /* safe */
+		  result[ind+len-1] = *key + '@';
+		  ind += len;
+		  key++;
+		  keylen--;
+	       }
+	     else
+	       {
+		  result[ind++] = *key++;
+		  keylen--;
+	       }
+	  }
+	else
+	  {
+	     int len = strlen(_(KeyNames[i]));
+	     if (len > maxlen-ind)
+	       break; /* not enough memory */
+	     strcpy(result+ind, _(KeyNames[i])); /* safe */
+	     ind += len;
+	     key += EscapeSequences[i][0];
+	     keylen -= EscapeSequences[i][0];
+	  }
+     }
+
+   if (keylen)
+     return NULL;
+
+   result[ind] = 0;
+   return result;
+}
+/*}}}*/
+
+/* Returns the corresponding key sequence for a human-friendly representation.
+ * Please note that the result is stored in a local static variable and may be
+ * NULL if the sequence would be too large. */
+char *slrn_help_string_to_keyseq (char *s) /*{{{*/
+{
+   /* We're using a nul-terminated representation here */
+   static char result [SLANG_MAX_KEYMAP_KEY_SEQ+1];
+   int ind = 0;
+   int slen = strlen(s);
+
+   while (slen && (ind <= SLANG_MAX_KEYMAP_KEY_SEQ))
+     {
+	char *end;
+	if ((*s == '<') && (NULL != (end = slrn_strbyte (s,'>'))))
+	  {
+	     int i;
+	     int len = end-s-1;
+	     for (i = 0; i < NUMBER_OF_KEYNAMES; i++)
+	       {
+		  if ((KeyNameLengths[i] == len+2) &&
+		      !slrn_case_strncmp (s+1,
+					  KeyNames[i]+1, len))
+		    break;
+	       }
+	     if (i < NUMBER_OF_KEYNAMES)
+	       {
+		  if (EscapeSequences[i][0] > SLANG_MAX_KEYMAP_KEY_SEQ - ind)
+		    break; /* not enough memory */
+		  strncpy(result+ind, EscapeSequences[i]+1, EscapeSequences[i][0]);
+		  ind += EscapeSequences[i][0];
+		  s += len+2;
+		  slen -= len+2;
+		  continue;
+	       }
+	  }
+	/* by default, just copy the character */
+	result[ind++] = *s++;
+	slen--;
+     }
+
+   if (slen)
+     return NULL;
+
+   result[ind] = 0;
+   return result;
+}
+/*}}}*/
