@@ -2,8 +2,8 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001-2003 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 1994, 1999, 2007-2012 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -24,9 +24,6 @@
 
 #include <stdio.h>
 
-/* rest of file inside this #if statement */
-#if SLRN_HAS_SLANG
-
 /*{{{ Include files */
 
 #if HAVE_STDLIB_H
@@ -52,19 +49,36 @@
 #include "print.h"
 #include "score.h"
 #include "snprintf.h"
-#include "mime.h"
 #include "version.h"
 #include "hooks.h"
+#include "help.h"
+#include "charset.h"
+#include "common.h"
+#include "strutil.h"
+#include "parse2822.h"
+#include "mime.h"
 
 /*}}}*/
 
 /*{{{ Public Global Variables */
 
-int Slrn_Use_Slang = 0;
-char *Slrn_Macro_Dir;
-
 /*}}}*/
 
+#if 0
+static void parse_rfc2822_intrin (char *header)
+{
+   char *errmsg;
+   char *map;
+
+   if (NULL == (map = slrn_parse_rfc2822_addr (header, &errmsg)))
+     {
+	SLang_verror (SL_Syntax_Error, "%s", errmsg);
+	return;
+     }
+   SLang_push_string (map);
+   SLfree (map);
+}
+#endif
 
 /*{{{ Screen update and message functions */
 
@@ -84,7 +98,7 @@ static void update (void) /*{{{*/
 static void free_argv_list (char **argv, unsigned int argc)
 {
    unsigned int i;
-   
+
    for (i = 0; i < argc; i++)
      slrn_free (argv[i]);
    slrn_free ((char *) argv);
@@ -106,13 +120,13 @@ static char **pop_argv_list (unsigned int *argcp)
      }
 
    argc = (unsigned int) n;
-      
+
    if (NULL == (argv = (char **) slrn_malloc (sizeof (char *) * (argc + 1), 1, 1)))
      {
-	SLang_Error = SL_MALLOC_ERROR;
+	SLang_set_error (SL_MALLOC_ERROR);
 	return NULL;
      }
-   
+
    argv [n] = NULL;
    i = argc;
    while (i != 0)
@@ -128,7 +142,7 @@ static char **pop_argv_list (unsigned int *argcp)
 
    return argv;
 }
-   
+
 static int interp_select_box (void) /*{{{*/
 {
    unsigned int n;
@@ -149,11 +163,11 @@ static int interp_select_box (void) /*{{{*/
 	free_argv_list (box.lines, n);
 	return -1;
      }
-   
+
    ret = slrn_select_box (&box);
    free_argv_list (box.lines, n);
    slrn_free (box.title);
-   
+
    return ret;
 }
 
@@ -169,24 +183,24 @@ static void select_list_box (void)
 
    if (-1 == SLang_pop_integer (&active))
      return;
-   
+
    argv = pop_argv_list (&argc);
    if (argv == NULL)
      return;
-   
+
    if (-1 == SLpop_string (&title))
      {
 	free_argv_list (argv, argc);
 	return;
      }
-   
+
    ret = slrn_select_list_mode (title, argc, argv, active - 1, 1, NULL);
    if (ret == -1)
      {
 	SLang_push_string ("");
 	return;
      }
-   
+
    SLang_push_string (argv[ret]);
 
    slrn_free (title);
@@ -197,30 +211,42 @@ static int popup_window (void)
 {
    char *title = NULL, *text = NULL;
    int retval = 0;
-   
+
    if ((-1 == SLpop_string (&text)) ||
        (-1 == SLpop_string (&title)))
      goto free_and_return;
-   
+
    retval = slrn_popup_win_mode (title, text);
-   
+
    free_and_return:
    slrn_free (title);
    slrn_free (text);
    return retval;
 }
 
-static int get_yesno_cancel (char *prompt)
+static int get_yesno_cancel (void)
 {
-   return slrn_get_yesno_cancel ("%s", prompt);
+   int dflt = 1;
+   char *prompt;
+
+   if (SLang_Num_Function_Args == 2)
+     {
+	if (-1 == SLang_pop_integer (&dflt))
+	  return -1;
+     }
+   if (-1 == SLang_pop_slstring (&prompt))
+     return -1;
+
+   dflt = slrn_get_yesno_cancel (dflt, "%s", prompt);
+   SLang_free_slstring (prompt);
+   return dflt;
 }
 
 static int get_response (char *choices, char *prompt)
 {
-   return slrn_get_response (choices, "%s", prompt);
+   return slrn_get_response (choices, choices, "%s", prompt);
 }
 
-   
 static void tt_send (char *s)
 {
    if (Slrn_Batch == 0)
@@ -230,6 +256,12 @@ static void tt_send (char *s)
      }
 }
 
+static void set_display_state (int *onoff)
+{
+   int state = (*onoff  ? (SLRN_TTY_INIT | SLRN_SMG_INIT) : 0);
+   slrn_set_display_state (state);
+}
+
 /*}}}*/
 
 /*{{{ File functions */
@@ -243,30 +275,22 @@ static void make_home_filename (char *name) /*{{{*/
 
 /*}}}*/
 
-static int evalfile (char *file)
-{
-   if (-1 == slrn_eval_slang_file (file))
-     return 0;
-
-   return 1;
-}
-
-
 int slrn_eval_slang_file (char *name) /*{{{*/
 {
+#if 0
    char file [SLRN_MAX_PATH_LEN];
-   
+
    if (Slrn_Macro_Dir != NULL)
      {
 	int n = 0;
 	char dir[SLRN_MAX_PATH_LEN];
-	
+
 	while (1)
 	  {
 	     if (-1 == SLextract_list_element (Slrn_Macro_Dir, n, ',',
-						   file, sizeof (file)))
+					       file, sizeof (file)))
 	       break;
-	     
+
 	     slrn_make_home_dirname (file, dir, sizeof (dir));
 	     if ((-1 != slrn_dircat (dir, name, file, sizeof (file)))
 		 && (1 == slrn_file_exists (file)))
@@ -279,25 +303,57 @@ int slrn_eval_slang_file (char *name) /*{{{*/
 	     n++;
 	  }
      }
-   
+
    slrn_make_home_filename (name, file, sizeof (file));
    slrn_message_now (_("loading %s"), file);
-   if (0 == SLang_load_file (file))
+   if (0 != SLang_load_file (file))
      return -1;
+
+   return 0;
+#else
+
+   if (0 != SLang_load_file (name))
+     return -1;
+
+   return 0;
+#endif
+}
+
+int slrn_set_macro_dir (char *dir)
+{
+   if (-1 == SLang_run_hooks ("slrn_set_macro_dir_hook", 1, dir))
+     return -1;
+
    return 0;
 }
 
+char *slrn_get_macro_dir (void)
+{
+   char *str;
+
+   if (1 != SLang_run_hooks ("slrn_get_macro_dir_hook", 0))
+     return NULL;
+
+   if (-1 == SLpop_string (&str))
+     return NULL;
+
+   return str;
+}
+
 /*}}}*/
 
-
-
 /*}}}*/
 
 /*{{{ Set/Get Variable Functions */
 
 static void set_string_variable (char *s1, char *s2) /*{{{*/
 {
-   if (-1 == slrn_set_string_variable (s1, s2))
+   char *charset = Slrn_Display_Charset;
+
+   if (SLinterp_is_utf8_mode ())
+     charset = "utf-8";
+
+   if (-1 == slrn_set_string_variable (s1, s2, charset))
      slrn_error (_("%s is not a valid variable name."), s1);
 }
 
@@ -313,29 +369,20 @@ static void set_integer_variable (char *s1, int *val) /*{{{*/
 
 static void get_variable_value (char *name) /*{{{*/
 {
-   char **s;
-   int *ip;
-   int type;
-   
-   if (-1 == slrn_get_variable_value (name, &type, &s, &ip))
+   char *s;
+   int i;
+   SLtype type;
+
+   if (-1 == slrn_get_variable_value (name, &type, &s, &i))
+     return;
+
+   if (type == SLANG_STRING_TYPE)
      {
-	slrn_error (_("%s is not a valid variable name."), name);
+	(void) SLang_push_malloced_string (s);
 	return;
      }
-   
-   if (type == STRING_TYPE)
-     {
-	char *str;
-	if ((s == NULL) || (*s == NULL)) str = "";
-	else str = *s;
-	SLang_push_string (str);
-     }
-   else if (type == INT_TYPE)
-     {
-	int i;
-	if (ip == NULL) i = 0; else i = *ip;
-	SLang_push_integer (i);
-     }
+
+   (void) SLang_push_integer (i);
 }
 
 /*}}}*/
@@ -354,125 +401,48 @@ static char *interp_setlocale (int *category, char *locale) /*{{{*/
 
 /*}}}*/
 
-static void set_utf8_conversion_table (void) /*{{{*/
-{
-#if SLANG_VERSION < 10400
-   slrn_error (_("To use this feature, please update s-lang and recompile slrn."));
-#else
-# if ! SLRN_HAS_MIME
-   slrn_error (_("This version of slrn lacks MIME support."));
-# else
-   SLang_Array_Type *at;
-   int rows;
-   
-   if (-1 == SLang_pop_array_of_type (&at, SLANG_INT_TYPE))
-     {
-	slrn_error (_("Array of integer expected."));
-	return;
-     }
-   
-   if ((at->num_dims != 2) || (at->dims[0] != 2))
-     {
-	slrn_error (_("Two-dimensional array with two columns expected."));
-	goto free_and_return;
-     }
-   
-   slrn_free (Slrn_Utf8_Table);
-   Slrn_Utf8_Table = NULL;
-   
-   if ((rows = at->dims[1]) == 0)
-     goto free_and_return; /* no data, so set default (iso-8859-1) */
-   
-   if (NULL == (Slrn_Utf8_Table = slrn_malloc (65536, 1, 0)))
-     {
-	slrn_error (_("Could not get memory for conversion table."));
-	goto free_and_return;
-     }
-   
-   while (rows-- != 0)
-     {
-	int dims[2];
-	int unicode, local;
-	
-	dims[0] = 0;
-	dims[1] = rows;
-	(void) SLang_get_array_element (at, dims, &unicode);
-	
-	if (((unicode != 0) && (unicode < 128)) || (unicode > 65535))
-	  {
-	     slrn_error (_("Unicode value in row %d out of range."), rows);
-	     goto free_and_return;
-	  }
-	
-	dims[0] = 1;
-	(void) SLang_get_array_element (at, dims, &local);
-	
-	if ((local < 1) || (local > 255))
-	  {
-	     slrn_error (_("Local charset value in row %d out of range."),
-			 rows);
-	     goto free_and_return;
-	  }
-	
-	Slrn_Utf8_Table[unicode] = (char) local;
-     }
-   
-   free_and_return:
-   SLang_free_array (at);
-# endif /* SLRN_HAS_MIME */
-#endif /* SLANG_VERSION */
-}
-/*}}}*/
-
 static void generic_set_regexp (SLRegexp_Type **regexp_table) /*{{{*/
 {
-#if SLANG_VERSION < 10400
-   slrn_error (_("To use this feature, please update s-lang and recompile slrn."));
-#else
    SLang_Array_Type *at;
    SLcmd_Cmd_Table_Type cmd_table;
    int argc, i;
-   
+
    if (-1 == SLang_pop_array_of_type (&at, SLANG_STRING_TYPE))
      {
 	slrn_error (_("Array of string expected."));
 	return;
      }
-   
+
    if (at->num_dims != 1)
      {
 	slrn_error (_("One-dimensional array expected."));
 	goto free_and_return;
      }
-   
+
    argc = at->dims[0];
-   
+
    if ((argc<1) || (argc > SLRN_MAX_REGEXP))
      {
 	slrn_error (_("Array must contain at least one, at most %d elements."),
 		    SLRN_MAX_REGEXP);
 	goto free_and_return;
      }
-   
+
    if (NULL == (cmd_table.string_args = (char**) slrn_malloc
 		((argc+1)*sizeof(char*), 0, 0)))
-     {
-	slrn_error (_("Failed to allocate memory."));
-	goto free_and_return;
-     }
-   
+     goto free_and_return;
+
    for (i = 0; i < argc; i++)
      {
 	(void) SLang_get_array_element (at, &i, &(cmd_table.string_args[i+1]));
      }
-   
+
    slrn_generic_regexp_fun (argc+1, &cmd_table, regexp_table);
-   
+
    slrn_free ((char*)cmd_table.string_args);
-   
+
    free_and_return:
    SLang_free_array (at);
-#endif   
 }/*}}}*/
 
 static void set_ignore_quotes (void) /*{{{*/
@@ -498,10 +468,10 @@ static void set_strip_was_regexp (void) /*{{{*/
 
 static char *get_server_name (void) /*{{{*/
 {
-   if ((NULL == Slrn_Server_Obj) 
+   if ((NULL == Slrn_Server_Obj)
        || (NULL == Slrn_Server_Obj->sv_name))
      return "";
-   
+
    return Slrn_Server_Obj->sv_name;
 }
 
@@ -512,32 +482,29 @@ static void quit (int *code) /*{{{*/
 }
 
 /*}}}*/
-   
+
 /*{{{ Keyboard related functions */
 
 static void definekey (char *fun, char *key, char *map) /*{{{*/
 {
    SLKeyMap_List_Type *kmap;
-   
+   char *convkey = slrn_help_string_to_keyseq(key);
+   if (convkey == NULL)
+     convkey = key;
+
    if (!strcmp (map, "readline"))
      map = "ReadLine";
-   
+
    if (NULL == (kmap = SLang_find_keymap (map)))
      {
 	slrn_error (_("definekey: no such keymap."));
 	return;
      }
-   
-   if ((kmap == Slrn_Keymap_RLI->keymap) &&
-       (NULL == SLang_find_key_function(fun, kmap)))
-     {
-	SLKeymap_Function_Type *tmp = kmap->functions;
-	kmap->functions = Slrn_Custom_Readline_Functions;
-	SLang_define_key (key, fun, kmap);
-	kmap->functions = tmp;
-     }
+
+   if (kmap == SLrline_get_keymap (Slrn_Keymap_RLI))
+     (void) slrn_rline_setkey (convkey, fun, kmap);
    else
-     SLang_define_key (key, fun, kmap);
+     (void) SLang_define_key (convkey, fun, kmap);
 }
 
 /*}}}*/
@@ -545,17 +512,21 @@ static void definekey (char *fun, char *key, char *map) /*{{{*/
 static void undefinekey (char *key, char *map) /*{{{*/
 {
    SLKeyMap_List_Type *kmap;
-   
+
+   char *convkey = slrn_help_string_to_keyseq(key);
+   if (convkey == NULL)
+     convkey = key;
+
    if (!strcmp (map, "readline"))
      map = "ReadLine";
-   
+
    if (NULL == (kmap = SLang_find_keymap (map)))
      {
 	error (_("undefinekey: no such keymap."));
 	return;
      }
-   
-   SLang_undefine_key (key, kmap);
+
+   SLang_undefine_key (convkey, kmap);
 }
 
 /*}}}*/
@@ -567,17 +538,19 @@ static void generic_read_mini (int mode, char *prompt, char *dfl, char *init) /*
 {
    char str[SLRL_DISPLAY_BUFFER_SIZE];
    int ret;
+   int point;
 
    strncpy (str, init, sizeof (str));
    str[sizeof(str) - 1] = 0;
-   
+
    switch (mode)
      {
       case PROMPT_NO_ECHO:
 	ret = slrn_read_input_no_echo (prompt, dfl, str, 0, 0);
 	break;
       case PROMPT_FILENAME:
-	ret = slrn_read_filename (prompt, dfl, str, 0, 0);
+	point = (int) (slrn_basename (str) - str);
+	ret = slrn_read_filename (prompt, dfl, str, 0, point);
 	break;
       case PROMPT_VARIABLE:
 	ret = slrn_read_variable (prompt, dfl, str, 0, 0);
@@ -585,7 +558,7 @@ static void generic_read_mini (int mode, char *prompt, char *dfl, char *init) /*
       default:
 	ret = slrn_read_input (prompt, dfl, str, 0, 0);
      }
-   
+
    if (-1 == ret)
      {
 	error (_("Quit!"));
@@ -623,12 +596,22 @@ static void read_mini_variable (char *prompt, char *dfl, char *init) /*{{{*/
 
 static int read_mini_integer (char *prompt, int *dfl) /*{{{*/
 {
-   int status, retval;
-   status = slrn_read_integer (prompt, dfl, &retval);
-   if (-1 == status)
-     error (_("Quit!"));
+   NNTP_Artnum_Type x;
+   int ix;
+
+   x = *dfl;
+   if (-1 == slrn_read_artnum_int (prompt, &x, &x))
+     {
+	error (_("Quit!"));
+	return -1;
+     }
+
+   ix = (int) x;
+   if (x != ix)
+     error (_("Number exceeds the size of an integer"));
+
    slrn_clear_message ();
-   return retval;
+   return ix;
 }
 /*}}}*/
 
@@ -644,7 +627,7 @@ static int get_prefix_arg (void) /*{{{*/
 #define NO_PREFIX_ARGUMENT -1
 
    return Slrn_Prefix_Arg_Ptr ? *Slrn_Prefix_Arg_Ptr : NO_PREFIX_ARGUMENT;
-} 
+}
 
 /*}}}*/
 
@@ -652,15 +635,14 @@ static void reset_prefix_arg (void) /*{{{*/
 {
    Slrn_Prefix_Arg_Ptr = NULL;
 }
-  
-/*}}}*/
 
+/*}}}*/
 
 static int check_tty (void)
 {
    if (Slrn_TT_Initialized & SLRN_TTY_INIT)
      return 0;
-   
+
    error (_("Terminal not initialized."));
    return -1;
 }
@@ -669,7 +651,7 @@ static int input_pending (int *tsecs)
 {
    if (check_tty ())
      return 0;
-   
+
    return SLang_input_pending (*tsecs);
 }
 
@@ -677,7 +659,7 @@ static int getkey (void)
 {
    if (check_tty ())
      return 0;
-   
+
    return SLang_getkey ();
 }
 
@@ -742,7 +724,7 @@ static void pipe_article_cmd (char *cmd) /*{{{*/
 {
    if (-1 == check_article_mode ())
      return;
-   
+
    (void) slrn_pipe_article_to_cmd (cmd);
 }
 
@@ -752,7 +734,7 @@ static int save_current_article (char *file) /*{{{*/
 {
    if (-1 == check_article_mode ())
      return -1;
-   
+
    return slrn_save_current_article (file);
 }
 
@@ -763,14 +745,14 @@ static int generic_search_article (char *str, int is_regexp, /*{{{*/
 {
    Slrn_Article_Line_Type *l;
    char *ptr;
-   
+
    if (-1 == check_article_mode ())
      return 0;
-   
+
    l = slrn_search_article (str, &ptr, is_regexp, 1, dir);
    if (l == NULL)
      return 0;
-   
+
    (void) SLang_push_string (l->buf);
    return 1;
 }
@@ -817,7 +799,7 @@ static char *extract_article_header (char *h) /*{{{*/
 {
    unsigned int len;
    char buf[128];
-   
+
    if (-1 == check_article_mode ())
      return NULL;
 
@@ -828,7 +810,7 @@ static char *extract_article_header (char *h) /*{{{*/
 	h = slrn_cur_extract_header (buf, len + 2);
      }
    else h = NULL;
-   
+
    if (h == NULL) h = "";
 
    return h;
@@ -840,7 +822,7 @@ static char *extract_current_article_header (char *h) /*{{{*/
 {
    unsigned int len;
    char buf[128];
-   
+
    if (-1 == check_article_mode ())
      return NULL;
    if (NULL == Slrn_Current_Article)
@@ -869,7 +851,7 @@ static char *generic_article_as_string (int use_orig) /*{{{*/
    unsigned int len;
    Slrn_Article_Line_Type *l;
    char *s, *s1;
-   
+
    if (Slrn_Current_Article == NULL)
      return "";
 
@@ -880,36 +862,36 @@ static char *generic_article_as_string (int use_orig) /*{{{*/
      {
 	char *buf;
 	Slrn_Article_Line_Type *next = l->next;
-	
+
 	buf = l->buf;
 	if (l->flags & WRAPPED_LINE) buf++;   /* skip space */
-	
+
 	len += strlen (buf);
-	
+
 	if ((next == NULL) || (0 == (next->flags & WRAPPED_LINE)))
 	  len++;
 	l = next;
      }
-   
+
    if (NULL == (s = (char *) SLMALLOC (len + 1)))
      {
-	SLang_Error = SL_MALLOC_ERROR;
+	SLang_set_error (SL_MALLOC_ERROR);
 	return NULL;
      }
-   
+
    l = use_orig ? Slrn_Current_Article->raw_lines : Slrn_Current_Article->lines;
    s1 = s;
-   
+
    while (l != NULL)
      {
 	char *buf;
 	Slrn_Article_Line_Type *next = l->next;
-	
+
 	buf = l->buf;
 	if (l->flags & WRAPPED_LINE) buf++;   /* skip space */
-	
+
 	while (*buf != 0) *s1++ = *buf++;
-	       
+
 	if ((next == NULL) || (0 == (next->flags & WRAPPED_LINE)))
 	  *s1++ = '\n';
 	l = next;
@@ -935,7 +917,7 @@ static void sort_articles (void) /*{{{*/
 {
    if (-1 == check_article_mode ())
      return;
-   
+
    slrn_sort_headers ();
 }
 
@@ -984,7 +966,7 @@ static int article_line_up (int *num) /*{{{*/
    if ((-1 == check_article_mode ())
        || (*num <= 0))
      return 0;
-   
+
    return slrn_art_lineup_n ((unsigned int)*num);
 }
 /*}}}*/
@@ -994,7 +976,7 @@ static int article_line_down (int *num) /*{{{*/
    if ((-1 == check_article_mode ())
        || (*num <= 0))
      return 0;
-   
+
    return slrn_art_linedn_n ((unsigned int)*num);
 }
 /*}}}*/
@@ -1082,7 +1064,7 @@ static int header_down (int *num) /*{{{*/
    if ((-1 == check_article_mode ())
        || (*num <= 0))
      return 0;
-   
+
    return slrn_header_down_n (*num, 0);
 }
 
@@ -1093,7 +1075,7 @@ static int header_up (int *num) /*{{{*/
    if ((-1 == check_article_mode ())
        || (*num <= 0))
      return 0;
-   
+
    return slrn_header_up_n (*num, 0);
 }
 
@@ -1103,7 +1085,7 @@ static int header_next_unread (void)
 {
    if (-1 == check_article_mode ())
      return -1;
-   
+
    return slrn_next_unread_header (1);
 }
 
@@ -1122,7 +1104,7 @@ static void set_header_flags (int *flagsp) /*{{{*/
    if ((-1 == check_article_mode ())
        || (Slrn_Current_Header == NULL))
      return;
-   
+
    slrn_set_header_flags (Slrn_Current_Header, (unsigned int) *flagsp);
 }
 
@@ -1161,10 +1143,10 @@ static int get_header_number (void)
 
 static int get_header_tag_number (void)
 {
-   if ((Slrn_Current_Header == NULL) 
+   if ((Slrn_Current_Header == NULL)
        || (0 == (Slrn_Current_Header->flags & HEADER_NTAGGED)))
      return 0;
-   
+
    return (int) Slrn_Current_Header->tag_number;
 }
 
@@ -1201,26 +1183,30 @@ static int re_header_search (char *pat, unsigned int offset, int dir) /*{{{*/
 {
    SLRegexp_Type *re;
    Slrn_Header_Type *h = Slrn_Current_Header;
-   
+   int status;
+
    if ((-1 == check_article_mode ())
        || (h == NULL)
        || (NULL == (re = slrn_compile_regexp_pattern (pat))))
      return 0;
-   
+
+   status = 0;
    while (h != NULL)
      {
 	if (NULL != slrn_regexp_match (re, *(char **) ((char *)h + offset)))
 	  {
 	     slrn_goto_header (h, 0);
-	     return 1;
+	     status = 1;
+	     break;
 	  }
-	
+
 	if (dir > 0)
 	  h = h->next;
 	else
 	  h = h->prev;
      }
-   return 0;   
+   SLregexp_free (re);
+   return status;
 }
 
 /*}}}*/
@@ -1231,7 +1217,6 @@ static int re_subject_search_forward (char *pat)
    return re_header_search (pat, (char *) &h.subject - (char *)&h, 1);
 }
 
-
 static int re_subject_search_backward (char *pat)
 {
    Slrn_Header_Type h;
@@ -1280,10 +1265,10 @@ static int is_group_mode (void)
 static int group_down_n (int *np)
 {
    int n = *np;
-   
+
    if (-1 == check_group_mode ())
      return 0;
-   
+
    if (n < 0) return slrn_group_up_n (-n);
    return slrn_group_down_n (n);
 }
@@ -1291,10 +1276,10 @@ static int group_down_n (int *np)
 static int group_up_n (int *np)
 {
    int n = *np;
-   
+
    if (-1 == check_group_mode ())
      return 0;
-   
+
    if (n < 0) return slrn_group_down_n (-n);
    return slrn_group_up_n (n);
 }
@@ -1310,7 +1295,7 @@ static char *current_group_name (void)
 {
    if (Slrn_Group_Current_Group == NULL)
      return "";
-   
+
    return Slrn_Group_Current_Group->group_name;
 }
 
@@ -1326,25 +1311,25 @@ static int select_group (void)
 {
    if (-1 == check_group_mode ())
      return -1;
-   
+
    return slrn_group_select_group ();
 }
 
 static int get_group_flags (void)
-{	
+{
    if (Slrn_Group_Current_Group == NULL)
      return 0;
-   
+
    return Slrn_Group_Current_Group->flags & GROUP_HARMLESS_FLAGS_MASK;
 }
 
 static void set_group_flags (int *flagsp)
 {
    unsigned int flags;
-   
+
    if (Slrn_Group_Current_Group == NULL)
      return;
-   
+
    Slrn_Group_Current_Group->flags &= ~GROUP_HARMLESS_FLAGS_MASK;
    flags = ((unsigned int) *flagsp) & GROUP_HARMLESS_FLAGS_MASK;
    Slrn_Group_Current_Group->flags |= flags;
@@ -1380,7 +1365,7 @@ static char *get_bg_color (char *name)
 
 static FILE *Log_File_Ptr;
 static void close_log_file (void)
-{	
+{
    if (Log_File_Ptr != NULL)
      {
 	(void) fclose (Log_File_Ptr);
@@ -1400,12 +1385,22 @@ static void open_log_file (char *file)
 static void log_message (char *buf)
 {
    FILE *fp;
-   
+
+   if (Slrn_Debug_Fp != NULL)
+     {
+	(void) fputs (buf, Slrn_Debug_Fp);
+	(void) fflush (Slrn_Debug_Fp);
+     }
+
    fp = Log_File_Ptr;
-   if (fp == NULL) fp = stderr;
-   
-   fputs (buf, fp);
-   fflush (fp);
+   if (fp == NULL)
+     {
+	if (Slrn_TT_Initialized & SLRN_SMG_INIT)
+	  return;
+	fp = stderr;
+     }
+   (void) fputs (buf, fp);
+   (void) fflush (fp);
 }
 
 static void message_now (char *s)
@@ -1447,14 +1442,35 @@ static int locate_header_by_msgid (char *msgid, int *qs)
    return 1;
 }
 
-static void replace_article_cmd (char *str)
+static void replace_article_internal (int cooked)
 {
-   if ((-1 == check_article_mode ())
-       || (Slrn_Current_Header == NULL))
+   int handle_mime = 0;
+   char *str;
+
+   if (SLang_Num_Function_Args == 2)
+     {
+	if (-1 == SLang_pop_integer (&handle_mime))
+	  return;
+     }
+   if (-1 == SLang_pop_slstring (&str))
      return;
 
-   if (-1 == slrn_string_to_article (str))
+   if ((0 == check_article_mode ())
+       && (Slrn_Current_Header != NULL)
+       && (-1 == slrn_string_to_article (str, handle_mime, cooked)))
      slrn_error (_("Could not replace article with given string."));
+
+   SLang_free_slstring (str);
+}
+
+static void replace_cooked_article_cmd (void)
+{
+   replace_article_internal (1);
+}
+
+static void replace_article_cmd (void)
+{
+   replace_article_internal (0);
 }
 
 static int is_article_visible (void)
@@ -1468,26 +1484,138 @@ static int is_article_visible (void)
 static void reload_scorefile (int *apply_now) /*{{{*/
 {
    char file[SLRN_MAX_PATH_LEN];
-   
+
    if (Slrn_Score_File == NULL)
      return;
-   
+
    slrn_make_home_filename (Slrn_Score_File, file, sizeof (file));
 
    if (Slrn_Scorefile_Open != NULL)
      slrn_free (Slrn_Scorefile_Open);
    Slrn_Scorefile_Open = slrn_safe_strmalloc (file);
-   
+
    if (is_article_mode() && *apply_now)
      slrn_apply_scores (*apply_now);
 }
 
 /*}}}*/
 
+static int do_register_unregister_hook (int reg)
+{
+   SLang_Name_Type *func;
+   char *name;
+   int status;
+
+   if (NULL == (func = SLang_pop_function ()))
+     return -1;
+
+   if (-1 == SLang_pop_slstring (&name))
+     {
+	SLang_free_function (func);
+	return -1;
+     }
+
+   if (reg)
+     status = slrn_register_hook (name, func);
+   else
+     status = slrn_unregister_hook (name, func);
+
+   SLang_free_function (func);
+   SLang_free_slstring (name);
+   return status;
+}
+
+static int register_hook (void)
+{
+   return do_register_unregister_hook (1);
+}
+
+static int unregister_hook (void)
+{
+   return do_register_unregister_hook (0);
+}
+
+static void rfc1522_decode_string (void)
+{
+   char *s;
+
+   if (-1 == SLpop_string (&s))
+     return;
+
+   if (-1 == slrn_rfc1522_decode_string (&s, 0))
+     {
+        SLfree (s);
+	return;
+     }
+   (void) SLang_push_malloced_string (s);   /* frees s also */
+}
+
+static void decode_string (char *(*f)(char *))
+{
+   char *str, *strend;
+   SLang_BString_Type *b;
+
+   if (-1 == SLpop_string (&str))
+     return;
+
+   if (NULL == (strend = (*f)(str)))
+     {
+	SLfree (str);
+	return;
+     }
+   *strend = 0;			       /* \0 terminate */
+
+   b = SLbstring_create_malloced ((unsigned char *)str, (unsigned int)(strend - str), 1);
+   if (b == NULL)		       /* str is freed upon error */
+     return;
+
+   (void) SLang_push_bstring (b);
+   SLbstring_free (b);
+}
+
+static void decode_base64 (void)
+{
+   decode_string (&slrn_decode_base64);
+}
+static void decode_qp (void)
+{
+   decode_string (&slrn_decode_qp);
+}
+
+static void charset_convert_string (char *str, char *from, char *to, int *testp)
+{
+   char *new_str, *strmax;
+
+   strmax = str + strlen (str);
+
+   new_str = slrn_convert_string (from, str, strmax, to, *testp);
+   if (new_str == NULL)
+     {
+	(void) SLang_push_null ();
+	return;
+     }
+   (void) SLang_push_malloced_string (new_str);
+}
+
+static void get_charset_intrin (char *name)
+{
+   (void) SLang_push_string (slrn_get_charset (name));   /* NULL ok */
+}
+
+#define S SLANG_STRING_TYPE
+#define I SLANG_INT_TYPE
 static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
 {
+   /* MAKE_INTRINSIC_S("parse_rfc2822", parse_rfc2822_intrin, SLANG_VOID_TYPE), */
+   MAKE_INTRINSIC_0("get_charset", get_charset_intrin, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_4("charset_convert_string", charset_convert_string, SLANG_VOID_TYPE, S, S, S, I),
+   MAKE_INTRINSIC_I("set_display_state", set_display_state, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("rfc1522_decode_string", rfc1522_decode_string, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("decode_base64_string", decode_base64, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("decode_qp_string", decode_qp, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("headers_hidden_mode", slrn_is_hidden_headers_mode, SLANG_INT_TYPE),
-   MAKE_INTRINSIC_S("replace_article", replace_article_cmd, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("replace_article", replace_article_cmd, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("replace_cooked_article", replace_cooked_article_cmd, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("message_now", message_now, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("set_visible_headers", set_visible_headers, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("get_visible_headers", get_visible_headers, SLANG_STRING_TYPE),
@@ -1529,7 +1657,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_SS("get_response", get_response, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("get_select_box_response", interp_select_box, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("get_variable_value", get_variable_value, SLANG_VOID_TYPE),
-   MAKE_INTRINSIC_S("get_yes_no_cancel", get_yesno_cancel, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("get_yes_no_cancel", get_yesno_cancel, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("getkey", getkey, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("goto_num_tagged_header", slrn_goto_num_tagged_header, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("group_down_n", group_down_n, SLANG_INT_TYPE),
@@ -1566,7 +1694,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_SSS("read_mini_filename", read_mini_filename, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SSS("read_mini_variable", read_mini_variable, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SI("read_mini_integer", read_mini_integer, SLANG_INT_TYPE),
-   MAKE_INTRINSIC_SS("register_hook", slrn_register_hook_by_name, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("register_hook", register_hook, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("reload_scorefile", reload_scorefile, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_I("request_body", request_body, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("save_current_article", save_current_article, SLANG_INT_TYPE),
@@ -1593,7 +1721,6 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_0("set_strip_re_regexp", set_strip_re_regexp, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("set_strip_sig_regexp", set_strip_sig_regexp, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("set_strip_was_regexp", set_strip_was_regexp, SLANG_VOID_TYPE),
-   MAKE_INTRINSIC_0("set_utf8_conversion_table", set_utf8_conversion_table, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("get_group_order", slrn_intr_get_group_order, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("set_group_order", slrn_intr_set_group_order, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_IS("setlocale", interp_setlocale, SLANG_STRING_TYPE),
@@ -1604,12 +1731,14 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_0("uncollapse_threads", uncollapse_threads, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SS("undefinekey", undefinekey, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_I("ungetkey", ungetkey, SLANG_VOID_TYPE),
-   MAKE_INTRINSIC_SS("unregister_hook", slrn_unregister_hook_by_name, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("unregister_hook", unregister_hook, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("update", update, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0(NULL, NULL, 0)
 };
 
 /*}}}*/
+#undef S
+#undef I
 
 static SLang_Intrin_Var_Type Intrin_Vars [] = /*{{{*/
 {
@@ -1624,10 +1753,8 @@ static SLang_Intrin_Var_Type Intrin_Vars [] = /*{{{*/
    MAKE_VARIABLE("HEADER_LOW_SCORE", &Interp_Header_Low_Score, SLANG_INT_TYPE, 1),
    MAKE_VARIABLE("HEADER_READ", &Interp_Header_Read, SLANG_INT_TYPE, 1),
    MAKE_VARIABLE("HEADER_TAGGED", &Interp_Header_Tagged, SLANG_INT_TYPE, 1),
-   MAKE_VARIABLE("SCREEN_HEIGHT", &SLtt_Screen_Rows, SLANG_INT_TYPE, 1),
-   MAKE_VARIABLE("SCREEN_WIDTH", &SLtt_Screen_Cols, SLANG_INT_TYPE, 1),
-   MAKE_VARIABLE("_slrn_version", &Slrn_Version_Number, SLANG_INT_TYPE, 1),
-   MAKE_VARIABLE("_slrn_version_string", &Slrn_Version, SLANG_STRING_TYPE, 1),
+   MAKE_VARIABLE("_slrn_version", &Slrn_Version, SLANG_INT_TYPE, 1),
+   MAKE_VARIABLE("_slrn_version_string", &Slrn_Version_String, SLANG_STRING_TYPE, 1),
 #ifdef HAVE_SETLOCALE
 # ifdef LC_CTYPE
    MAKE_VARIABLE("LC_CTYPE", &Interp_Lc_Ctype, SLANG_INT_TYPE, 1),
@@ -1649,36 +1776,82 @@ static int interp_system (char *s) /*{{{*/
 
 /*}}}*/
 
+static int add_intrinsic_variables (void)
+{
+   /* This is necessary because of the windows DLL limitations */
+   if ((-1 == SLadd_intrinsic_variable ("SCREEN_HEIGHT", (VOID_STAR)&SLtt_Screen_Rows, SLANG_INT_TYPE, 1))
+       || ((-1 == SLadd_intrinsic_variable ("SCREEN_WIDTH", (VOID_STAR)&SLtt_Screen_Cols, SLANG_INT_TYPE, 1)))
+      )
+     return -1;
+
+   return 0;
+}
+
+static int load_startup_file (void)
+{
+   char *dir;
+
+   if (NULL == (dir = getenv (ENV_SLRN_SLANG_DIR)))
+     dir = SLRN_SLANG_DIR;
+
+   if ((dir == NULL) || (*dir == 0))
+     {
+	slrn_error (_("The SLRN_SLANG_DIR variable is either NULL or empty"));
+	return -1;
+     }
+
+   if (-1 == SLpath_set_load_path (dir))
+     return -1;
+
+   if (-1 == SLang_load_file ("slrn.sl"))
+     {
+	slrn_error (_("Configuration error: Unable to load startup file"));
+
+	if (SLang_get_error () == SL_Open_Error)
+	  {
+	     slrn_error (_("The hardcoded load path is set to: %s"), SLRN_SLANG_DIR);
+	     slrn_error (_("You make need to set the %s environment variable."), ENV_SLRN_SLANG_DIR);
+	  }
+	return -1;
+     }
+
+   return 0;
+}
+
 int slrn_init_slang (void) /*{{{*/
 {
-   Slrn_Use_Slang = 0;
    if ((-1 == SLang_init_slang ())
        || (-1 == SLang_init_slmath ())
+       || (-1 == SLang_init_array ())
        || (-1 == SLang_init_posix_process ())
        || (-1 == SLang_init_posix_dir ())
        || (-1 == SLang_init_stdio ())
        || (-1 == SLang_init_posix_io ())
        || (-1 == SLang_init_ospath ())
-       || (-1 == SLang_init_slassoc ())
-#if SLANG_VERSION >= 10400
        || (-1 == SLang_init_import ()) /* enable dynamic linking */
-#endif
-       
        /* Now add intrinsics for this application */
        || (-1 == SLadd_intrin_fun_table(Slrn_Intrinsics, NULL))
-       || (-1 == SLadd_intrin_var_table(Intrin_Vars, NULL)))
+       || (-1 == SLadd_intrin_var_table(Intrin_Vars, NULL))
+       || (-1 == add_intrinsic_variables ()))
      return -1;
-   
+
    SLadd_intrinsic_function ("system", (FVOID_STAR) interp_system, SLANG_INT_TYPE, 1, SLANG_STRING_TYPE);
-   SLadd_intrinsic_function ("evalfile", (FVOID_STAR) evalfile, SLANG_INT_TYPE, 1, SLANG_STRING_TYPE);
+   /* SLadd_intrinsic_function ("evalfile", (FVOID_STAR) evalfile, SLANG_INT_TYPE, 1, SLANG_STRING_TYPE); */
+
+   (void) SLang_load_file_verbose (1);
 
    SLang_Error_Hook = error;
-
-   Slrn_Use_Slang = 1;
    SLang_User_Clear_Error = slrn_clear_message;
    SLang_Exit_Error_Hook = slrn_va_exit_error;
    SLang_Dump_Routine = log_message;
    SLang_VMessage_Hook = slrn_va_message;
+
+   if (Slrn_Debug_Fp != NULL)
+     SLang_Traceback = 1;
+
+   if (-1 == load_startup_file ())
+     return -1;
+
    return 0;
 }
 
@@ -1689,5 +1862,3 @@ int slrn_reset_slang (void)
    close_log_file ();
    return 0;
 }
-
-#endif
