@@ -1,7 +1,7 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 1994, 1999, 2007-2012 John E. Davis <jed@jedsoft.org>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -14,11 +14,11 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 extern int slrn_init_slang (void);
 extern int slrn_eval_slang_file (char *);
-extern int Slrn_Use_Slang;
-extern char *Slrn_Macro_Dir;
+extern int slrn_set_macro_dir (char *dir);
+extern char *slrn_get_macro_dir (void);
 extern int slrn_reset_slang (void);
