@@ -1,8 +1,8 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 1994, 1999, 2007-2012 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -35,7 +35,7 @@
 #include <slang.h>
 #include "jdmacros.h"
 
-#if SLRN_USE_SLTCP 
+#if SLRN_USE_SLTCP
 # include "sltcp.h"
 #else
 # include "clientlib.h"
@@ -46,9 +46,12 @@
 #include "nntplib.h"
 #include "ttymsg.h"
 #include "snprintf.h"
+#include "server.h"
+#include "strutil.h"
+#include "common.h"
 
 void (*NNTP_Connection_Lost_Hook) (NNTP_Type *);
-int (*NNTP_Authorization_Hook) (char *, char **, char **);
+int (*NNTP_Authorization_Hook) (char *, int, char **, char **);
 int NNTP_Try_Authentication = 0;
 
 int Slrn_Broken_Xref;
@@ -58,6 +61,9 @@ static int _nntp_connect_server (NNTP_Type *);
 extern int Slrn_Force_Authentication;
 extern FILE *Slrn_Debug_Fp; /* we need this here when compiling slrnpull */
 
+/* Note: nntp_allocate_nntp returns a pointer to a static object.  Eventually,
+ * it should be changed to return a dynamic one.
+ */
 static NNTP_Type *nntp_allocate_nntp (void)
 {
    static NNTP_Type nn;
@@ -77,6 +83,12 @@ static void _nntp_deallocate_nntp (NNTP_Type *s)
 {
    if (s == NULL) return;
 
+   if (s->tcp != NULL)
+     sltcp_close (s->tcp);
+
+   /* Until nntp_allocate_nntp is modified to create a dynamic object, just
+    * zero the memory here.
+    */
    memset ((char *) s, 0, sizeof (NNTP_Type));
 }
 
@@ -91,7 +103,6 @@ static int check_connect_lost_hook (NNTP_Type *s)
 
    return -1;
 }
-   
 
 int nntp_fgets_server (NNTP_Type *s, char *buf, unsigned int len)
 {
@@ -117,17 +128,17 @@ int nntp_fputs_server (NNTP_Type *s, char *buf)
 {
    if ((s == NULL) || (s->init_state == 0))
      return -1;
-   
+
    if (-1 == sltcp_fputs (s->tcp, buf))
      {
 	(void) nntp_disconnect_server (s);
 	(void) check_connect_lost_hook (s);
 	return -1;
      }
-   
+
    if (Slrn_Debug_Fp != NULL)
      fprintf (Slrn_Debug_Fp, ">%s\n", buf);
-   
+
    return 0;
 }
 
@@ -135,14 +146,14 @@ int nntp_write_server (NNTP_Type *s, char *buf, unsigned int n)
 {
    if ((s == NULL) || (s->init_state == 0))
      return -1;
-   
+
    if (n != sltcp_write (s->tcp, buf, n))
      {
 	(void) nntp_disconnect_server (s);
 	(void) check_connect_lost_hook (s);
 	return -1;
      }
-   
+
    if (Slrn_Debug_Fp != NULL)
      {
 	unsigned int i;
@@ -151,7 +162,7 @@ int nntp_write_server (NNTP_Type *s, char *buf, unsigned int n)
 	  fputc (buf[i], Slrn_Debug_Fp);
 	fputc ('\n', Slrn_Debug_Fp);
      }
-   
+
    return 0;
 }
 
@@ -161,7 +172,7 @@ int nntp_gets_server (NNTP_Type *s, char *buf, unsigned int len)
      return -1;
 
    len = strlen (buf);
-   
+
    /* Update bytes received */
    s->number_bytes_received += len;
 
@@ -196,7 +207,6 @@ static void _nntp_error_response (NNTP_Type *s, char *fmt)
    slrn_error (_("Reason: %s"), s->rspbuf);
 }
 
-
 static int _nntp_try_parse_timeout (char *str)
 {
    /* I know of only two timeout responses:
@@ -208,6 +218,7 @@ static int _nntp_try_parse_timeout (char *str)
    /* Some servers now have NLS, so this might fail.
     * I'm trying to handle that case in nntp_server_cmd.
     */
+#if SLANG_VERSION < 20000
    static SLRegexp_Type re;
    unsigned char compiled_pattern_buf[256];
 
@@ -219,8 +230,19 @@ static int _nntp_try_parse_timeout (char *str)
    (void) SLang_regexp_compile (&re);
    if (NULL == SLang_regexp_match ((unsigned char *) str, strlen (str), &re))
      return -1;
-
    return 0;
+#else
+   int status = -1;
+   SLRegexp_Type *re = SLregexp_compile ("time.*out", SLREGEXP_CASELESS);
+   if (re != NULL)
+     {
+	if (NULL != SLregexp_match (re, str, strlen (str)))
+	  status = 0;
+
+	SLregexp_free (re);
+     }
+   return status;
+#endif
 }
 
 int nntp_get_server_response (NNTP_Type *s)
@@ -241,7 +263,7 @@ int nntp_get_server_response (NNTP_Type *s)
 	if (-1 == _nntp_try_parse_timeout (s->rspbuf))
 	  break;
 	/* Drop */
-	
+
       case 0:			       /* invalid code */
       case ERR_GOODBYE:
 	nntp_disconnect_server (s);
@@ -257,7 +279,7 @@ int nntp_get_server_response (NNTP_Type *s)
 static int _nntp_reconnect (NNTP_Type *s)
 {
    nntp_disconnect_server (s);
-   
+
    if (-1 == check_connect_lost_hook (s))
      return -1;
 
@@ -272,7 +294,7 @@ static int _nntp_reconnect (NNTP_Type *s)
 	s->flags |= NNTP_RECONNECT_OK;
 	return -1;
      }
-   
+
    if ((s->group_name[0] != 0)
        && (-1 == nntp_server_vcmd (s, "GROUP %s", s->group_name)))
      {
@@ -325,7 +347,7 @@ int nntp_server_cmd (NNTP_Type *s, char *cmd)
 	     if ((code == ERR_NOAUTH) && NNTP_Try_Authentication)
 	       {
 		  NNTP_Try_Authentication = 0;
-		  if (-1 == nntp_authorization (s))
+		  if (-1 == nntp_authorization (s, 1))
 		    return -1;
 		  tried_auth = 1;
 		  continue;
@@ -357,7 +379,7 @@ int nntp_server_vcmd (NNTP_Type *s, char *fmt, ...)
    va_list ap;
 
    va_start (ap, fmt);
-   slrn_vsnprintf (buf, sizeof (buf), fmt, ap);
+   (void) SLvsnprintf (buf, sizeof (buf), fmt, ap);
    va_end (ap);
 
    return nntp_server_cmd (s, buf);
@@ -369,7 +391,7 @@ int nntp_start_server_vcmd (NNTP_Type *s, char *fmt, ...)
    va_list ap;
 
    va_start (ap, fmt);
-   slrn_vsnprintf (buf, sizeof (buf), fmt, ap);
+   (void) SLvsnprintf (buf, sizeof (buf), fmt, ap);
    va_end (ap);
 
    return nntp_start_server_cmd (s, buf);
@@ -390,27 +412,27 @@ int nntp_close_server (NNTP_Type *s)
     * of the *server_cmd functions.
     */
    (void) nntp_puts_server (s, "QUIT");
-   
+
    if (Slrn_Debug_Fp != NULL)
      fputs ("!Closing the server connection.\n", Slrn_Debug_Fp);
-   
-   (void) sltcp_close (s->tcp);
-
-   s->tcp = NULL;
-   s->init_state = 0;
 
    _nntp_deallocate_nntp (s);
 
    return 0;
 }
 
-int nntp_authorization (NNTP_Type *s)
+int nntp_authorization (NNTP_Type *s, int auth_reqd)
 {
-   char *name, *pass;
+   char *name = NULL, *pass = NULL;
+   int status = 0;
 
-   if ((NNTP_Authorization_Hook == NULL) ||
-       (-1 == (*NNTP_Authorization_Hook) (s->name, &name, &pass)) ||
-       ((name == NULL) || (pass == NULL)))
+   if (NNTP_Authorization_Hook != NULL)
+     status = (*NNTP_Authorization_Hook) (s->name, auth_reqd, &name, &pass);
+
+   if ((auth_reqd == 0) && (status == 0))
+     return 0;			       /* not needed and info not present */
+
+   if ((status == -1) || (name == NULL) || (pass == NULL))
      {
 	slrn_exit_error (_("Authorization needed, but could not determine username / password."));
 	return -1;
@@ -434,7 +456,19 @@ int nntp_authorization (NNTP_Type *s)
 	if (s->can_post == 1)
 	  break;
 	/* This is the only obvious way to find out whether we are
-	 * able to post after successful authentication. */
+	 * able to post after successful authentication.
+	 *
+	 * However, at least one server will disconnect if an attempt
+	 * is made to post an empty article.  Try to avoid this by
+	 * looking at the response message.  Unfortunately, this method
+	 * is not very robust.
+	 */
+	if (NULL != strstr (s->rspbuf, "Posting Allowed"))
+	  {
+	     s->can_post = 1;
+	     break;
+	  }
+
 	if (-1 == nntp_post_cmd (s))
 	  return -1;
 	if (s->code == CONT_POST)
@@ -467,7 +501,7 @@ static int _nntp_connect_server (NNTP_Type *s)
    s->init_state = 1;
    s->number_bytes_received = 0;
    NNTP_Try_Authentication = 2;
-   
+
    /* Read logon message. */
    switch (nntp_get_server_response (s))
      {
@@ -483,19 +517,25 @@ static int _nntp_connect_server (NNTP_Type *s)
 	goto failed;
      }
 
+   /* Try to identify INN */
+   if (NULL != strstr (s->rspbuf, "INN"))
+     s->sv_id = SERVER_ID_INN;
+   else
+     s->sv_id = SERVER_ID_UNKNOWN;
+
    if ((-1 == nntp_server_cmd (s, "MODE READER"))
        || (ERR_ACCESS == s->code))
      goto failed;
-   
+
    if (s->code == OK_NOPOST)
      s->can_post = 0;
-   
-   if (Slrn_Force_Authentication && (-1 == nntp_authorization (s)))
+
+   if (-1 == nntp_authorization (s, Slrn_Force_Authentication))
      return -1;
 
    slrn_message_now (_("Connected to host.  %s"),
 		     (s->can_post ? _("Posting ok.") : _("Posting NOT ok.")));
-   
+
    return 0;
 
    failed:
@@ -503,7 +543,7 @@ static int _nntp_connect_server (NNTP_Type *s)
    _nntp_error_response (s, _("Failed to initialize server"));
    if (Slrn_Debug_Fp != NULL)
      fputs ("!Server connect failed.\n", Slrn_Debug_Fp);
-   
+
    (void) sltcp_close (s->tcp);
    s->tcp = NULL;
    return -1;
@@ -564,7 +604,7 @@ char *nntp_get_server_name (void)
    if (NULL != (host = _nntp_getserverbyfile(NNTPSERVER_FILE)))
      return host;
 
-   slrn_stderr_strcat ("\n", _("You need to set the NNTPSERVER environment variable to your server name."), "\n", NULL);
+   slrn_stderr_strcat ("\n", _("You should set the NNTPSERVER environment variable to your server name."), "\n", NULL);
 #ifdef VMS
    slrn_stderr_strcat (_("Example: $ define/job NNTPSERVER my.news.server"), "\n", NULL);
 #else
@@ -575,18 +615,19 @@ char *nntp_get_server_name (void)
 		       _("Example (sh) : NNTPSERVER='my.news.server' && export NNTPSERVER"), "\n", NULL);
 # endif
 #endif
-   return NULL;
+   slrn_stderr_strcat (_("For now, I'm going to try \"localhost\" as the default..."), "\n", NULL);
+   return "localhost";
 }
 
-
-/* In general, host has the form: "address:port".  If port is non-negative
- * use its value despite the value coded in the hostname.
- */
+/* In general, host has the form: "address:port" or "[address]:port" (to
+ * support IPv6 literal addresses).  If port is non-negative use its value
+ * despite the value coded in the hostname. */
 static int _nntp_setup_host_and_port (char *host, int port, NNTP_Type *s)
 {
    char *a, *b, *bmax;
    int def_port = 119;
    char *def_service = "nntp";
+   int quoteaddr=0;
 
    a = host;
    if (0 == strncmp (a, "snews://", 8))
@@ -602,8 +643,21 @@ static int _nntp_setup_host_and_port (char *host, int port, NNTP_Type *s)
 
    b = s->host;
    bmax = b + NNTP_MAX_HOST_LEN;
-   while ((*a != 0) && (*a != ':') && (b < bmax))
-     *b++ = *a++;
+   while ((*a != 0) && ((*a != ':') || (quoteaddr)) && (b < bmax))
+     {
+	if(*a == '[')
+	  {
+	     quoteaddr=1;
+	     a++;
+	  }
+	else if(*a == ']')
+	  {
+	     quoteaddr=0;
+	     a++;
+	  }
+	else
+	  *b++ = *a++;
+     }
    *b = 0;
    slrn_strncpy (s->name, host, NNTP_MAX_HOST_LEN);
 
@@ -615,9 +669,9 @@ static int _nntp_setup_host_and_port (char *host, int port, NNTP_Type *s)
 
    if (port <= 0)
      {
-	if (*a == ':')
-	  port = atoi (a + 1);
-	else if (-1 == (port = sltcp_map_service_to_port (def_service)))
+	if (((*a != ':') || (!(port = atoi (a+1))
+	     && (-1 == (port = sltcp_map_service_to_port (a+1)))))
+	    && (-1 == (port = sltcp_map_service_to_port (def_service))))
 	  port = def_port;
      }
    s->port = port;
@@ -625,7 +679,6 @@ static int _nntp_setup_host_and_port (char *host, int port, NNTP_Type *s)
    return 0;
 }
 
-
 NNTP_Type *nntp_open_server (char *host, int port)
 {
    NNTP_Type *s;
@@ -651,14 +704,28 @@ NNTP_Type *nntp_open_server (char *host, int port)
    return s;
 }
 
+/* This function removes the leading '.' from dot-escaped lines */
 int nntp_read_line (NNTP_Type *s, char *buf, unsigned int len)
 {
    if (-1 == nntp_gets_server (s, buf, len))
      return -1;
 
-   if ((buf[0] == '.') && (buf[1] == 0))
-     return 0;
-
+   if (buf[0] == '.')
+     {
+	char ch = buf[1];
+	if (ch == 0)
+	  return 0;
+	if (ch == '.')
+	  {
+	     char *b = buf;
+	     do
+	       {
+		  ch = b[1];
+		  *b++ = ch;
+	       }
+	     while (ch != 0);
+	  }
+     }
    return 1;
 }
 
@@ -704,10 +771,10 @@ int nntp_check_connection (NNTP_Type *s)
 void nntp_disconnect_server (NNTP_Type *s)
 {
    if (s == NULL) return;
-   
+
    if (Slrn_Debug_Fp != NULL)
      fputs ("!Disconnecting from server.\n", Slrn_Debug_Fp);
-   
+
    sltcp_close_socket (s->tcp);
 }
 
@@ -722,7 +789,8 @@ static int _nntp_probe_server (NNTP_Type *s, char *cmd)
    return 1;
 }
 
-#define PROBE_XCMD(s, var,cmd) (((var) != -1) ? (var) : ((var) = _nntp_probe_server ((s),(cmd))))
+#define PROBE_XCMD(s, var, cmd) \
+   (((var) != -1) ? (var) : ((var) = _nntp_probe_server ((s),(cmd))))
 
 int nntp_has_cmd (NNTP_Type *s, char *cmd)
 {
@@ -736,23 +804,35 @@ int nntp_has_cmd (NNTP_Type *s, char *cmd)
 
 	ret = nntp_server_cmd (s, cmd);
 
-	if (ret == ERR_COMMAND)
-	  s->can_xhdr = 0;
-	else if ((ret == OK_HEAD) &&
-		 (1 == nntp_read_line (s, buf, sizeof (buf))))
+	switch (ret)
 	  {
-	     char *p = buf;
-	     nntp_discard_output (s);
-	     while (*p && (*p != ' ')) p++;
-	     while (*p == ' ') p++;
-	     if (!strcmp (p, "(none)"))
-	       s->can_xhdr = 0;
-	     else
-	       s->can_xhdr = 1;
+	   default:
+	     s->can_xhdr = 0;
+	     break;
+
+	   case ERR_CMDSYN:
+	   case ERR_NCING:
+	     s->can_xhdr = 1;
+	     break;
+
+	   case ERR_COMMAND:
+	   case ERR_ACCESS:
+	     s->can_xhdr = 0;
+	     break;
+
+	   case OK_HEAD:
+	     s->can_xhdr = 1;
+	     if (1 == nntp_read_line (s, buf, sizeof (buf)))
+	       {
+		  char *p = buf;
+		  nntp_discard_output (s);
+		  while (*p && (*p != ' ')) p++;
+		  while (*p == ' ') p++;
+		  if (!strcmp (p, "(none)"))
+		    s->can_xhdr = 0;
+	       }
+	     break;
 	  }
-	else if (strlen (cmd) == 4)
-	  s->can_xhdr = 1;
-	
 	return s->can_xhdr;
      }
 
@@ -765,58 +845,57 @@ int nntp_has_cmd (NNTP_Type *s, char *cmd)
    return _nntp_probe_server (s, cmd);
 }
 
-static int _nntp_num_or_msgid_cmd (NNTP_Type *s, char *cmd, int n, char *msgid)
+static int _nntp_num_or_msgid_cmd (NNTP_Type *s, char *cmd, NNTP_Artnum_Type n, char *msgid)
 {
    if ((n != -1) && ((Slrn_Broken_Xref == 0) || (msgid == NULL)))
-     return nntp_server_vcmd (s, "%s %d", cmd, n);
+     return nntp_server_vcmd (s, ("%s " NNTP_FMT_ARTNUM), cmd, n);
+   else if (msgid == NULL)
+     return nntp_server_cmd(s, cmd);
    else
-     {
-	if (msgid == NULL) msgid = "";
-	return nntp_server_vcmd (s, "%s %s", cmd, msgid);
-     }
+     return nntp_server_vcmd (s, "%s %s", cmd, msgid);
 }
 
-int nntp_head_cmd (NNTP_Type *s, int n, char *msgid, int *real_id)
+int nntp_head_cmd (NNTP_Type *s, NNTP_Artnum_Type n, char *msgid, NNTP_Artnum_Type *real_id)
 {
    int status;
 
    status = _nntp_num_or_msgid_cmd (s, "HEAD", n, msgid);
    if ((status == OK_HEAD) && (real_id != NULL))
-     *real_id = atoi(s->rspbuf + 4);
+     *real_id = NNTP_STR_TO_ARTNUM(s->rspbuf + 4);
    return status;
 }
 
-int nntp_xover_cmd (NNTP_Type *s, int min, int max)
+int nntp_xover_cmd (NNTP_Type *s, NNTP_Artnum_Type min, NNTP_Artnum_Type max)
 {
-   return nntp_server_vcmd (s, "XOVER %d-%d", min, max);
+   return nntp_server_vcmd (s, "XOVER " NNTP_FMT_ARTRANGE, min, max);
 }
 
-int nntp_xhdr_cmd (NNTP_Type *s, char *field, int min, int max)
+int nntp_xhdr_cmd (NNTP_Type *s, char *field, NNTP_Artnum_Type min, NNTP_Artnum_Type max)
 {
-   return nntp_server_vcmd (s, "XHDR %s %d-%d", field, min, max);
+   return nntp_server_vcmd (s, "XHDR %s " NNTP_FMT_ARTRANGE, field, min, max);
 }
 
-int nntp_article_cmd (NNTP_Type *s, int n, char *msgid)
+int nntp_article_cmd (NNTP_Type *s, NNTP_Artnum_Type n, char *msgid)
 {
    return _nntp_num_or_msgid_cmd (s, "ARTICLE", n, msgid);
 }
 
-int nntp_next_cmd (NNTP_Type *s, int *n)
+int nntp_next_cmd (NNTP_Type *s, NNTP_Artnum_Type *n)
 {
    int status;
 
    if ((OK_NEXT == (status = nntp_server_cmd (s, "NEXT")))
        && (n != NULL))
-     *n = atoi (s->rspbuf + 4);
+     *n = NNTP_STR_TO_ARTNUM (s->rspbuf + 4);
 
    return status;
 }
 
-int nntp_select_group (NNTP_Type *s, char *name, int *minp, int *maxp)
+int nntp_select_group (NNTP_Type *s, char *name, NNTP_Artnum_Type *minp, NNTP_Artnum_Type *maxp)
 {
-   int estim;
-   int min, max;
-   
+   NNTP_Artnum_Type estim;
+   NNTP_Artnum_Type min, max;
+
    switch (nntp_server_vcmd (s, "GROUP %s", name))
      {
       case -1:
@@ -824,7 +903,7 @@ int nntp_select_group (NNTP_Type *s, char *name, int *minp, int *maxp)
 
       case OK_GROUP:
 
-	if (3 != sscanf(s->rspbuf + 4, "%d %d %d", &estim, &min, &max))
+	if (3 != sscanf (s->rspbuf + 4, NNTP_FMT_ARTNUM_3, &estim, &min, &max))
 	  return -1;
 
 	if (minp != NULL) *minp = min;
@@ -832,7 +911,7 @@ int nntp_select_group (NNTP_Type *s, char *name, int *minp, int *maxp)
 
 	slrn_strncpy (s->group_name, name, NNTP_MAX_GROUP_NAME_LEN);
 	break;
-	
+
       case ERR_ACCESS:
       default:
 	break;
@@ -846,27 +925,27 @@ int nntp_select_group (NNTP_Type *s, char *name, int *minp, int *maxp)
 int nntp_refresh_groups (NNTP_Type *s, Slrn_Group_Range_Type *gr, int n)
 {
    int reconnect, i, prev = 0, max_tries = 3;
-   
+
    start_over:
-   
+
    /* If we might still need authentication, use nntp_select_group first. */
    if ((NNTP_Try_Authentication == 2) && n)
      {
 	int status;
 	NNTP_Try_Authentication = 1;
-	
+
 	status = nntp_select_group (s, gr->name, &(gr->min), &(gr->max));
 	if (status == -1) return -1;
 	else if (status == ERR_NOGROUP)
 	  gr->min = -1;
 	n--; gr++;
      }
-   
+
    max_tries--;
    reconnect = s->flags & NNTP_RECONNECT_OK;
    s->flags &= ~NNTP_RECONNECT_OK;
    /* disallow reconnect for the moment as we batch commands */
-   
+
    i = prev;
    while (i < n)
      {
@@ -882,7 +961,8 @@ int nntp_refresh_groups (NNTP_Type *s, Slrn_Group_Range_Type *gr, int n)
    i = prev;
    while (i < n)
      {
-	int estim;
+	NNTP_Artnum_Type estim;
+
 	prev = i;
 	switch (nntp_get_server_response (s))
 	  {
@@ -891,7 +971,7 @@ int nntp_refresh_groups (NNTP_Type *s, Slrn_Group_Range_Type *gr, int n)
 	     if ((max_tries == 0) || (-1 == _nntp_reconnect (s)))
 	       return -1;
 	     goto start_over;
-	     
+
 	   case ERR_NOAUTH:
 	     s->flags |= reconnect;
 	     while (++i < n)
@@ -899,16 +979,16 @@ int nntp_refresh_groups (NNTP_Type *s, Slrn_Group_Range_Type *gr, int n)
 	     if (NNTP_Try_Authentication)
 	       {
 		  NNTP_Try_Authentication = 0;
-		  if (-1 == nntp_authorization (s))
+		  if (-1 == nntp_authorization (s, 1))
 		    return -1;
 		  max_tries++;
 		  goto start_over;
 	       }
 	     else
 	       return -1;
-	     
+
 	   case OK_GROUP:
-	     if (3 != sscanf (s->rspbuf + 4, "%d %d %d", &estim,
+	     if (3 != sscanf (s->rspbuf + 4, NNTP_FMT_ARTNUM_3, &estim,
 			      &(gr[i].min), &(gr[i].max)))
 	       {
 		  s->flags |= reconnect;
@@ -918,7 +998,7 @@ int nntp_refresh_groups (NNTP_Type *s, Slrn_Group_Range_Type *gr, int n)
 	       }
 	     slrn_strncpy (s->group_name, gr[i].name, NNTP_MAX_GROUP_NAME_LEN);
 	     break;
-	     
+
 	   case ERR_FAULT:
 	   case OK_GOODBYE: /* SN */
 	     if (max_tries == 2) /* might be a timeout we didn't recognize */
@@ -929,7 +1009,7 @@ int nntp_refresh_groups (NNTP_Type *s, Slrn_Group_Range_Type *gr, int n)
 		  goto start_over;
 	       }
 	     /* Fall through */
-	     
+
 	   default:
 	     gr[i].min = -1;
 	     break;
@@ -953,16 +1033,17 @@ int nntp_end_post (NNTP_Type *s)
    return nntp_get_server_response (s);
 }
 
-int nntp_xpat_cmd (NNTP_Type *s, char *hdr, int rmin, int rmax, char *pat)
+int nntp_xpat_cmd (NNTP_Type *s, char *hdr, NNTP_Artnum_Type rmin, NNTP_Artnum_Type rmax, char *pat)
 {
    if (0 == PROBE_XCMD(s, s->can_xpat, "XPAT"))
      return ERR_COMMAND;
 
-   return nntp_server_vcmd (s, "XPAT %s %d-%d *%s*", hdr, rmin, rmax, pat);
+   return nntp_server_vcmd (s, ("XPAT %s " NNTP_FMT_ARTRANGE " *%s*"),
+			    hdr, rmin, rmax, pat);
 }
 
 /* hdr should not include ':' */
-int nntp_one_xhdr_cmd (NNTP_Type *s, char *hdr, int num, char *buf, unsigned int buflen)
+int nntp_one_xhdr_cmd (NNTP_Type *s, char *hdr, NNTP_Artnum_Type num, char *buf, unsigned int buflen)
 {
    char tmpbuf[1024];
    int found;
@@ -977,7 +1058,7 @@ int nntp_one_xhdr_cmd (NNTP_Type *s, char *hdr, int num, char *buf, unsigned int
      {
 	char *b, ch;
 
-	if (-1 == nntp_server_vcmd (s, "XHDR %s %d", hdr, num))
+	if (-1 == nntp_server_vcmd (s, ("XHDR %s " NNTP_FMT_ARTNUM), hdr, num))
 	  return -1;
 
 	if (OK_HEAD != s->code)
@@ -1023,7 +1104,7 @@ int nntp_one_xhdr_cmd (NNTP_Type *s, char *hdr, int num, char *buf, unsigned int
 	char *b;
 
 	if (found
-	    || slrn_case_strncmp ((unsigned char *) tmpbuf, (unsigned char *) hdr, colon)
+	    || slrn_case_strncmp ( tmpbuf,  hdr, colon)
 	    || (tmpbuf[colon] != ':'))
 	  continue;
 
@@ -1062,7 +1143,7 @@ int nntp_listgroup (NNTP_Type *s, char *group)
    /*  OK_GROUP desired */
 }
 
-int nntp_body_cmd (NNTP_Type *s, int n, char *msgid)
+int nntp_body_cmd (NNTP_Type *s, NNTP_Artnum_Type n, char *msgid)
 {
    return _nntp_num_or_msgid_cmd (s, "BODY", n, msgid);
 }
@@ -1109,7 +1190,7 @@ char *nntp_read_and_malloc (NNTP_Type *s)
      {
 	if (mbuf == NULL)
 	  mbuf = slrn_strmalloc ("", 0);
-	
+
 	return mbuf;
      }
 
@@ -1121,25 +1202,25 @@ char *nntp_map_code_to_string (int code)
 {
    switch (code)
      {
-      case INF_HELP:	/* 100 */	
+      case INF_HELP:	/* 100 */
 	return _("Help text on way");
-      case INF_AUTH:	/* 180 */	
+      case INF_AUTH:	/* 180 */
 	return _("Authorization capabilities");
-      case INF_DEBUG:	/* 199 */	
+      case INF_DEBUG:	/* 199 */
 	return _("Debug output");
-      case OK_CANPOST:	/* 200 */	
+      case OK_CANPOST:	/* 200 */
 	return _("Hello; you can post");
-      case OK_NOPOST:	/* 201 */	
+      case OK_NOPOST:	/* 201 */
 	return _("Hello; you can't post");
-      case OK_SLAVE:	/* 202 */	
+      case OK_SLAVE:	/* 202 */
 	return _("Slave status noted");
-      case OK_GOODBYE:	/* 205 */	
+      case OK_GOODBYE:	/* 205 */
 	return _("Closing connection");
-      case OK_GROUP:	/* 211 */	
+      case OK_GROUP:	/* 211 */
 	return _("Group selected");
-      case OK_GROUPS:	/* 215 */	
+      case OK_GROUPS:	/* 215 */
 	return _("Newsgroups follow");
-      case OK_ARTICLE:	/* 220 */	
+      case OK_ARTICLE:	/* 220 */
 	return _("Article (head & body) follows");
       case OK_HEAD:	/* 221 */
 	return _("Head follows");
