@@ -1,8 +1,8 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 1994, 1999, 2007-2012 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -21,19 +21,22 @@
 #ifndef _SLRN_SERVER_H
 #define _SLRN_SERVER_H
 #include "nntpcodes.h"
+#include "ranges.h"
 
-typedef struct 
+typedef struct
 {
    int (*po_start)(void);
    int (*po_end)(void);
-   int (*po_printf)(char *, ...);
+   int (*po_printf)(char *, ...) ATTRIBUTE_PRINTF(1,2);
+   int (*po_vprintf)(const char *, va_list);
    int (*po_puts)(char *);
+   char * (*po_get_recom_id)(void);
    int po_can_post;
 } Slrn_Post_Obj_Type;
 
-typedef struct 
+typedef struct
 {
-   int (*sv_select_group) (char *, int *, int *);
+   int (*sv_select_group) (char *, NNTP_Artnum_Type *, NNTP_Artnum_Type *);
    int (*sv_refresh_groups) (Slrn_Group_Range_Type *, int);
    char * (*sv_current_group) (void);
    int (*sv_read_line) (char *, unsigned int);
@@ -43,11 +46,12 @@ typedef struct
     */
    void (*sv_reset)(void);
    int (*sv_initialize) (void);
-   int (*sv_select_article) (int, char *);
+   int (*sv_select_article) (NNTP_Artnum_Type, char *);
+   int (*sv_get_article_size) (NNTP_Artnum_Type);
    int (*sv_put_server_cmd) (char *, char *, unsigned int);
-   int (*sv_xpat_cmd) (char *, int, int, char *);
+   int (*sv_xpat_cmd) (char *, NNTP_Artnum_Type, NNTP_Artnum_Type, char *);
 
-   int (*sv_xhdr_command) (char *, int, char *, unsigned int);
+   int (*sv_xhdr_command) (char *, NNTP_Artnum_Type, char *, unsigned int);
 
    int (*sv_has_cmd) (char *);
    int (*sv_list) (char *);
@@ -58,28 +62,31 @@ typedef struct
    int sv_has_xhdr;
    int sv_has_xover;
    int sv_reset_has_xover;
-   /* if non-zero, sv_has_xover is set to 1 when entering a group.  
+   /* if non-zero, sv_has_xover is set to 1 when entering a group.
     * This is because some servers support XOVER but do not have overview
     * files for all groups.  See xover.c
     */
-   int (*sv_nntp_xover) (int, int);
-   int (*sv_nntp_xhdr) (char *, int, int);
-   int (*sv_nntp_head) (int, char *, int *);
-   int (*sv_nntp_next) (int *);
+   int (*sv_nntp_xover) (NNTP_Artnum_Type, NNTP_Artnum_Type);
+   int (*sv_nntp_xhdr) (char *, NNTP_Artnum_Type, NNTP_Artnum_Type);
+   int (*sv_nntp_head) (NNTP_Artnum_Type, char *, NNTP_Artnum_Type *);
+   int (*sv_nntp_next) (NNTP_Artnum_Type *);
 
    /* Returns number of bytes received.
     * If the int is non-zero, the counter is reset. */
    unsigned int (*sv_nntp_bytes) (int);
-   
+
+   /* Some server software has known bugs that we can work around. */
+#define SERVER_ID_UNKNOWN	0
+#define SERVER_ID_INN		1
+   int sv_id;
    char *sv_name;
-   
-} 
+}
 Slrn_Server_Obj_Type;
 
 extern Slrn_Server_Obj_Type *Slrn_Server_Obj;
 extern Slrn_Post_Obj_Type *Slrn_Post_Obj;
 
-extern int Slrn_Server_Min, Slrn_Server_Max;
+extern NNTP_Artnum_Type Slrn_Server_Min, Slrn_Server_Max;
 extern char *Slrn_Current_Group_Name;
 
 #if SLRN_HAS_NNTP_SUPPORT
@@ -111,7 +118,12 @@ extern char *Slrn_Nov_File;
 extern char *Slrn_Active_File;
 extern char *Slrn_ActiveTimes_File;
 extern char *Slrn_Newsgroups_File;
+extern char *Slrn_Overviewfmt_File;
 extern int Slrn_Spool_Check_Up_On_Nov;
+
+extern Slrn_Range_Type *slrn_spool_get_no_body_ranges (char *);
+extern Slrn_Range_Type *slrn_spool_get_requested_ranges (char *);
+extern int slrn_spool_set_requested_ranges (char*, Slrn_Range_Type*);
 #endif
 
 #if SLRN_HAS_PULL_SUPPORT
