@@ -2,8 +2,8 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001-2003  Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 1994, 1999, 2007-2012 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2001-2006  Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -52,12 +52,13 @@
 # include <sys/socket.h>
 #endif
 
-#if defined(__NT__) 
+#if defined(__NT__)
 # include <winsock.h>
 # define USE_WINSOCK_SLTCP	1
 #else
 # if defined(__WIN32__)
-#  define Win32_Winsock
+/* #  define Win32_Winsock */
+#  define __USE_W32_SOCKETS
 #  include <windows.h>
 #  define USE_WINSOCK_SLTCP	1
 # endif
@@ -69,7 +70,6 @@
 # include <netdb.h>
 #endif
 
-
 #ifdef HAVE_NETINET_IN_H
 # include <netinet/in.h>
 #endif
@@ -113,9 +113,13 @@ extern int h_errno;
 # include <gnutls/openssl.h>
 #else
 # if SLTCP_HAS_SSL_SUPPORT
-#  include <openssl/ssl.h>
-#  include <openssl/err.h>
-#  include <openssl/rand.h>
+#  if SLTCP_HAS_NSS_COMPAT
+#   include <nss_compat_ossl.h>
+#  else
+#   include <openssl/ssl.h>
+#   include <openssl/err.h>
+#   include <openssl/rand.h>
+#  endif
 # endif
 #endif
 
@@ -142,7 +146,6 @@ struct _SLTCP_Type
 #endif
 };
 
-
 /*}}}*/
 
 #if defined(__BEOS__) || defined(USE_WINSOCK_SLTCP)
@@ -162,17 +165,36 @@ int SLtcp_TimeOut_Secs = SLRN_SLTCP_TIMEOUT_SECS;
  * to get translated. */
 static int TCP_Verbose_Reporting = 0;
 
+void (*SLtcp_Verror_Hook) (char *, va_list);
+static void print_error (char *fmt, ...) SLATTRIBUTE_PRINTF(1,2);
+static void print_error (char *fmt, ...)
+{
+   va_list ap;
+
+   va_start (ap, fmt);
+   if (SLtcp_Verror_Hook != NULL)
+     (*SLtcp_Verror_Hook) (fmt, ap);
+   else
+     {
+	(void) vfprintf(stderr, fmt, ap);
+	(void) fputc ('\n', stderr);
+	fflush (stderr);
+     }
+   va_end (ap);
+}
+
+/* void (*SLtcp_Error_Hook) (char *, ...) SLATTRIBUTE_PRINTF(1,2); */
+
 static int sys_call_interrupted_hook (void) /*{{{*/
 {
    if (SLTCP_Interrupt_Hook == NULL)
      return 0;
-   
+
    return (*SLTCP_Interrupt_Hook) ();
 }
 
 /*}}}*/
 
-
 /* This function attempts to make a connection to a specified port on an
  * internet host.  It returns a socket descriptor upon success or -1
  * upon failure.
@@ -187,11 +209,11 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
    int tries = 1;
    int r;
    char portstr[6]; /* To pass a port number as a string */
-   
+
    /* We need to give a hint to getaddrinfo to get it to resolve a
     * numerical port number */
    struct addrinfo hint;
-   
+
    hint.ai_flags = 0;
    hint.ai_family = PF_UNSPEC;
    hint.ai_socktype = SOCK_STREAM;
@@ -200,25 +222,25 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
    hint.ai_addr = NULL;
    hint.ai_canonname = NULL;
    hint.ai_next = NULL;
-   
+
    do {
       snprintf(portstr, 6, "%i", port);
       if ((r = getaddrinfo(host, portstr, &hint, &res)) != 0) {
 	 if (TCP_Verbose_Reporting) {
-	    fprintf (stderr, "Error resolving %s (port %s): %s\n", host, portstr, gai_strerror(r));
+	    print_error ("Error resolving %s (port %s): %s\n", host, portstr, gai_strerror(r));
 	 }
 	 if (r == EAI_AGAIN) {
 	    slrn_sleep (1);
 	 };
       }
    } while (r && (tries++ <= 3));
-   
+
    if (r) {
-      fprintf (stderr, _("Failed to resolve %s\n"), host);
+      print_error (_("Failed to resolve %s\n"), host);
       return -1;
    }
    if (TCP_Verbose_Reporting) {
-      fprintf (stderr, "Successfully resolved %s\n", host);
+      print_error ("Successfully resolved %s\n", host);
    }
 
    ai = res;
@@ -226,52 +248,60 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
 	 * to get a socket connection */
       if (TCP_Verbose_Reporting) {
 	 struct sockaddr *a = ai->ai_addr;
+# ifdef HAVE_GETNAMEINFO
 	 static char buf[NI_MAXHOST];
 	 int l;
 	 if (a->sa_family == AF_INET) {
 	    l = sizeof(struct sockaddr_in);
-	    fprintf (stderr, "Address family: AF_INET\n");
+	    print_error ("Address family: AF_INET\n");
 	 } else {
 	    assert(a->sa_family == AF_INET6);
 	    l = sizeof(struct sockaddr_in6);
-	    fprintf (stderr, "Address family: AF_INET6\n");
+	    print_error ("Address family: AF_INET6\n");
 	 }
-# ifdef HAVE_GETNAMEINFO
 	 if (!getnameinfo(a, l, buf, NI_MAXHOST-1, NULL, 0, NI_NUMERICHOST)) {
-	    fprintf (stderr, "Will try with address %s", buf);
+	    print_error ("Will try with address %s", buf);
 	 } else {
-	    fprintf (stderr, "getnameinfo failed: %s\n", strerror(errno));
+	    print_error ("getnameinfo failed: %s\n", strerror(errno));
+	 }
+# else
+	 if (a->sa_family == AF_INET) {
+	    print_error ("Address family: AF_INET\n");
+	 } else {
+	    assert(a->sa_family == AF_INET6);
+	    print_error ("Address family: AF_INET6\n");
 	 }
 # endif /* HAVE_GETNAMEINFO */
       }
-      
+
       if ((fd = socket(ai->ai_family, SOCK_STREAM, 0)) == -1) {
 	 if (TCP_Verbose_Reporting) {
-	    fprintf (stderr, "Error creating socket: %s\n", strerror(errno));
+	    print_error ("Error creating socket: %s\n", strerror(errno));
 	 }
       } else {
 	 if (TCP_Verbose_Reporting) {
-	    fprintf (stderr, "Created socket; descriptor is = %i\n", fd);
+	    print_error ("Created socket; descriptor is = %i\n", fd);
 	 }
 	 if ((r = connect(fd, ai->ai_addr, ai->ai_addrlen)) == 0) {
 	    if (TCP_Verbose_Reporting) {
-	       fprintf (stderr, "Successfully connected\n");
+	       print_error ("Successfully connected\n");
 	    }
 	    connected = 1;
 	 } else {
 	    if (TCP_Verbose_Reporting) {
-	       fprintf (stderr, "Error connecting: %i, %s\n", errno, strerror(errno));
+	       print_error ("Error connecting: %i, %s\n", errno, strerror(errno));
 	    }
+	    (void) SLTCP_CLOSE (fd);
 	 }
       }
       ai = ai->ai_next;
    } while ((!connected) && (ai != NULL));
-   
+
    freeaddrinfo(res);
-   
+
    if (!connected)
-     fprintf(stderr, _("Unable to make connection. Giving up.\n"));
-   
+     print_error(_("Unable to make connection. Giving up.\n"));
+
    return connected ? fd : -1;
 #else
    /* IPv4-only implementation */
@@ -294,7 +324,7 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
      {
 	struct hostent *hp;
 	unsigned int max_retries = 3;
-	
+
 	while (NULL == (hp = gethostbyname (host)))
 	  {
 # ifdef TRY_AGAIN
@@ -305,10 +335,10 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
 		  continue;
 	       }
 # endif
-	     fprintf(stderr, _("%s: Unknown host.\n"), host);
+	     print_error(_("%s: Unknown host.\n"), host);
 	     return -1;
 	  }
-	
+
 # ifdef h_addr
 	h_addr_list = hp->h_addr_list;
 # else
@@ -319,12 +349,12 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
 	h_length = hp->h_length;
 	h_addr_type = hp->h_addrtype;
      }
-   else 
+   else
      {
 	h_addr_list = fake_h_addr_list;
 	h_addr_list [0] = (char *) &fake_addr;
 	h_addr_list [1] = NULL;
-	
+
 	h_length = sizeof(struct in_addr);
 	h_addr_type = AF_INET;
      }
@@ -338,23 +368,23 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
 	perror("socket");
 	return -1;
      }
-	
+
    not_connected = -1;
-   
-   while (not_connected 
-	  && (h_addr_list != NULL) 
+
+   while (not_connected
+	  && (h_addr_list != NULL)
 	  && (*h_addr_list != NULL))
      {
 	char *this_host;
-	
+
 	memcpy ((char *) &s_in.sin_addr, *h_addr_list, h_length);
 
 	this_host = (char *) inet_ntoa (s_in.sin_addr);
-	
-	if (TCP_Verbose_Reporting) fprintf (stderr, "trying %s\n", this_host);
-	
+
+	if (TCP_Verbose_Reporting) print_error ("trying %s\n", this_host);
+
 	not_connected = connect (s, (struct sockaddr *)&s_in, sizeof (s_in));
-	
+
 	if (not_connected == -1)
 	  {
 # ifdef EINTR
@@ -364,16 +394,16 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
 		    continue;
 	       }
 # endif
-	     fprintf (stderr, _("connection to %s, port %d:"), 
+	     print_error (_("connection to %s, port %d:"),
 		      (char *) this_host, port);
 	     perror ("");
 	  }
 	h_addr_list++;
      }
-   
-   if (not_connected) 
+
+   if (not_connected)
      {
-	fprintf(stderr, _("Unable to make connection. Giving up.\n"));
+	print_error(_("Unable to make connection. Giving up.\n"));
 	(void) SLTCP_CLOSE (s);
 	return -1;
      }
@@ -390,7 +420,7 @@ static void restore_sigint_handler (void (*f)(int), int call_it) /*{{{*/
 {
    if (f == SIG_ERR)
      return;
-   
+
    (void) SLsignal_intr (SIGINT, f);
 
    if (call_it)
@@ -399,7 +429,6 @@ static void restore_sigint_handler (void (*f)(int), int call_it) /*{{{*/
 
 /*}}}*/
 
-   
 static sigjmp_buf Sigint_Jmp_Buf;
 
 static void (*Old_Sigint_Handler) (int);
@@ -408,7 +437,7 @@ static volatile int Jump_In_Progress;
 static void sigint_handler (int sig) /*{{{*/
 {
    (void) sig;
-   
+
    if (Jump_In_Progress) return;
    Jump_In_Progress = 1;
 
@@ -422,7 +451,7 @@ static int get_tcp_socket (char *host, int port) /*{{{*/
    int fd;
 
    Old_Sigint_Handler = SIG_ERR;
-   
+
    Jump_In_Progress = 1;	       /* dont allow jump yet */
    if (0 != sigsetjmp (Sigint_Jmp_Buf, 1))   /* save signal mask */
      {
@@ -430,22 +459,22 @@ static int get_tcp_socket (char *host, int port) /*{{{*/
 	sys_call_interrupted_hook ();
 	return -1;
      }
-   
+
    Old_Sigint_Handler = SLsignal_intr (SIGINT, sigint_handler);
-   
-   if ((Old_Sigint_Handler == SIG_IGN) 
+
+   if ((Old_Sigint_Handler == SIG_IGN)
        || (Old_Sigint_Handler == SIG_DFL))
      {
 	restore_sigint_handler (Old_Sigint_Handler, 0);
 	Old_Sigint_Handler = SIG_ERR;
      }
-   
+
    Jump_In_Progress = 0;	       /* now allow the jump */
    fd = get_tcp_socket_1 (host, port);
    Jump_In_Progress = 1;	       /* don't allow jump */
-   
+
    restore_sigint_handler (Old_Sigint_Handler, 0);
-   
+
    return fd;
 }
 
@@ -474,47 +503,57 @@ static void deinit_ssl (void)
 static void dump_ssl_error_0 (void)
 {
    int err;
-   
+
    while (0 != (err = ERR_get_error()))
-     fprintf (stderr, "%s\n", ERR_error_string(err, 0));
+     print_error ("%s\n", ERR_error_string(err, 0));
 }
 
-unsigned long Fast_Random;
+#if !SLTCP_HAS_NSS_COMPAT
+static unsigned long Fast_Random;
 static unsigned long fast_random (void)
 {
    return (Fast_Random = Fast_Random * 69069U + 1013904243U);
 }
+#endif
 
 static int init_ssl_random (void)
 {
+#if !SLTCP_HAS_NSS_COMPAT
    time_t t;
    pid_t pid;
    unsigned int count;
 
    if (RAND_status ())
      return 0;
-   
+
    time (&t);
    pid = getpid ();
    RAND_seed ((unsigned char *) &pid, sizeof (time_t));
    RAND_seed ((unsigned char *) &t, sizeof (pid_t));
-   
+
    RAND_bytes ((unsigned char *) &Fast_Random, sizeof(long));
-   
+
    count = 0;
-   while ((count < 10000) 
+   while ((count < 10000)
 	  && (0 == RAND_status ()))
      {
 	unsigned long r = fast_random ();
 	RAND_seed (&r, sizeof (unsigned long));
      }
+#endif				       /* !SLTCP_HAS_NSS_COMPAT */
    if (RAND_status ())
      return 0;
-   
-   fprintf (stderr, _("Unable to generate enough entropy\n"));
+
+   print_error (_("Unable to generate enough entropy\n"));
    return -1;
 }
 
+#if SLTCP_HAS_GNUTLS_SUPPORT
+static void tls_log_func (int level, const char *str)
+{
+   print_error ("tls: level=%d: %s\n", level, str);
+}
+#endif
 
 static SSL *alloc_ssl (void)
 {
@@ -526,26 +565,32 @@ static SSL *alloc_ssl (void)
 
 	SSLeay_add_ssl_algorithms ();
 	c = SSL_CTX_new(SSLv23_client_method());
+	/* c = SSL_CTX_new(SSLv3_client_method()); */
 	SSL_load_error_strings ();
 	if (c == NULL)
 	  {
 	     dump_ssl_error_0 ();
-	     fprintf (stderr, _("SSL_CTX_new failed.\n"));
+	     print_error (_("SSL_CTX_new failed.\n"));
 	     return NULL;
 	  }
+	/* SSL_CTX_set_options (c, SSL_OP_NO_TLSv1); */
 	This_SSL_Ctx = c;
 	atexit (deinit_ssl);
-	
+
 	if (-1 == init_ssl_random ())
 	  return NULL;
      }
-   
+
+#if SLTCP_HAS_GNUTLS_SUPPORT
+   gnutls_global_set_log_function(tls_log_func);
+   gnutls_global_set_log_level(4711);
+#endif
    ssl = SSL_new (This_SSL_Ctx);
    if (ssl != NULL)
      return ssl;
-   
+
    dump_ssl_error_0 ();
-   fprintf (stderr, _("SSL_new failed\n"));
+   print_error (_("SSL_new failed\n"));
    return NULL;
 }
 
@@ -562,17 +607,17 @@ static void dump_ssl_error (SSL *ssl, char *msg, int status)
       case SSL_ERROR_NONE:
 	break;
       case SSL_ERROR_ZERO_RETURN:
-	fprintf (stderr, _("Unexpected error: SSL connection closed\n"));
+	print_error (_("Unexpected error: SSL connection closed\n"));
 	break;
       case SSL_ERROR_SYSCALL:
-	fprintf (stderr, _("System call failed: errno = %d\n"), errno);
+	print_error (_("System call failed: errno = %d\n"), errno);
 	break;
       case SSL_ERROR_SSL:
-	fprintf (stderr, _("Possible protocol error\n"));
+	print_error (_("Possible protocol error\n"));
 	break;
      }
    dump_ssl_error_0 ();
-   if (msg != NULL) fprintf (stderr, _("%s failed\n"), msg);
+   if (msg != NULL) print_error (_("%s failed\n"), msg);
 }
 #endif				       /* SLTCP_HAS_SSL_SUPPORT */
 
@@ -591,7 +636,7 @@ SLTCP_Type *sltcp_open_connection (char *host, int port, int use_ssl) /*{{{*/
 	if (NULL == (ssl = alloc_ssl ()))
 	  return NULL;
 #else
-	fprintf (stderr, _("\n\n*** This program does not support SSL\n"));
+	print_error (_("\n\n*** This program does not support SSL\n"));
 	return NULL;
 #endif
      }
@@ -599,20 +644,20 @@ SLTCP_Type *sltcp_open_connection (char *host, int port, int use_ssl) /*{{{*/
    tcp = (SLTCP_Type *) SLMALLOC (sizeof (SLTCP_Type));
    if (tcp == NULL)
      {
-	fprintf (stderr, _("Memory Allocation Failure.\n"));
+	print_error (_("Memory Allocation Failure.\n"));
 #if SLTCP_HAS_SSL_SUPPORT
 	dealloc_ssl (ssl);
 #endif
 	return NULL;
      }
    memset ((char *) tcp, 0, sizeof (SLTCP_Type));
-   
+
    tcp->tcp_fd = -1;
    tcp->tcp_read_ptr = tcp->tcp_read_ptr_max = tcp->tcp_read_buf;
    tcp->tcp_write_ptr = tcp->tcp_write_ptr_min = tcp->tcp_write_buf;
-   
+
    fd = get_tcp_socket (host, port);
-   if (fd == -1) 
+   if (fd == -1)
      {
 	SLFREE (tcp);
 #if SLTCP_HAS_SSL_SUPPORT
@@ -631,11 +676,11 @@ SLTCP_Type *sltcp_open_connection (char *host, int port, int use_ssl) /*{{{*/
    if (1 != SSL_set_fd (ssl, fd))
      {
 	/* Yuk.  SSL_set_fd returns 0 upon failure */
-	fprintf (stderr, _("SSL_set_fd failed\n"));
+	print_error (_("SSL_set_fd failed\n"));
 	sltcp_close (tcp);
 	return NULL;
      }
-   
+
    status = SSL_connect (ssl);
    if (status == 1)
      return tcp;
@@ -653,7 +698,7 @@ SLTCP_Type *sltcp_open_connection (char *host, int port, int use_ssl) /*{{{*/
 static int check_errno (void)
 {
 #ifdef EAGAIN
-   if (errno == EAGAIN) 
+   if (errno == EAGAIN)
      {
 	slrn_sleep (1);
 	return 0;
@@ -667,7 +712,7 @@ static int check_errno (void)
      }
 #endif
 #ifdef EINTR
-   if (errno == EINTR) 
+   if (errno == EINTR)
      {
 	if (-1 == sys_call_interrupted_hook ())
 	  return -1;
@@ -689,7 +734,7 @@ static unsigned int do_ssl_write (SLTCP_Type *tcp, unsigned char *buf, unsigned
    while (total != len)
      {
 	int nwrite;
-	
+
 	errno = 0;
 	nwrite = SSL_write (ssl, (char *) buf, (len - total));
 
@@ -699,7 +744,7 @@ static unsigned int do_ssl_write (SLTCP_Type *tcp, unsigned char *buf, unsigned
 	     tcp->bytes_out += total;
 	     continue;
 	  }
-	
+
 	switch (SSL_get_error (ssl, nwrite))
 	  {
 	   default:
@@ -711,7 +756,7 @@ static unsigned int do_ssl_write (SLTCP_Type *tcp, unsigned char *buf, unsigned
 	     if (-1 == sys_call_interrupted_hook ())
 	       return total;
 	     break;		       /* try again */
-	     
+
 	   case SSL_ERROR_SYSCALL:
 	     if (-1 == check_errno ())
 	       return total;
@@ -735,11 +780,11 @@ static unsigned int do_write (SLTCP_Type *tcp, unsigned char *buf, unsigned int
 #endif
    total = 0;
    fd = tcp->tcp_fd;
-   
+
    while (total != len)
      {
 	nwrite = SLTCP_WRITE (fd, (char *) buf, (len - total));
-	
+
 	if (nwrite == -1)
 	  {
 	     if (-1 == check_errno ())
@@ -749,7 +794,7 @@ static unsigned int do_write (SLTCP_Type *tcp, unsigned char *buf, unsigned int
 
 	total += (unsigned int) nwrite;
      }
-   
+
    tcp->bytes_out += total;
 
    return total;
@@ -768,18 +813,18 @@ int sltcp_flush_output (SLTCP_Type *tcp) /*{{{*/
 
    if ((tcp == NULL) || (-1 == (fd = tcp->tcp_fd)))
      return -1;
-   
+
    buf = tcp->tcp_write_ptr_min;
    n = (unsigned int) (tcp->tcp_write_ptr - buf);
-   
+
    nwrite = do_write (tcp, buf, n);
-   
+
    if (nwrite != n)
      {
 	tcp->tcp_write_ptr_min += nwrite;
 	return -1;
      }
-   
+
    tcp->tcp_write_ptr_min = tcp->tcp_write_ptr = tcp->tcp_write_buf;
    return 0;
 }
@@ -789,7 +834,7 @@ int sltcp_flush_output (SLTCP_Type *tcp) /*{{{*/
 int sltcp_close_socket (SLTCP_Type *tcp)
 {
    int status;
-   
+
    if ((tcp == NULL)
        || (tcp->tcp_fd == -1))
      return 0;
@@ -805,7 +850,7 @@ int sltcp_close_socket (SLTCP_Type *tcp)
 	     status = -1;
 	     break;
 	  }
-	
+
 	if (-1 == sys_call_interrupted_hook ())
 	  return -1;
      }
@@ -820,7 +865,7 @@ int sltcp_close (SLTCP_Type *tcp) /*{{{*/
    errno = 0;
 
    if (tcp == NULL) return -1;
-   
+
    if (-1 != tcp->tcp_fd)
      {
 	if (-1 == sltcp_flush_output (tcp))
@@ -829,7 +874,7 @@ int sltcp_close (SLTCP_Type *tcp) /*{{{*/
 	if (-1 == sltcp_close_socket (tcp))
 	  return -1;
      }
-   
+
    SLFREE (tcp);
    return 0;
 }
@@ -842,40 +887,40 @@ static int wait_for_input (SLTCP_Type *tcp) /*{{{*/
    fd_set fds;
    struct timeval tv;
    int fd = tcp->tcp_fd;
-   
+
    if (fd == -1)
      return -1;
-   
+
 #if SLTCP_HAS_SSL_SUPPORT
    if (tcp->ssl != NULL && SSL_pending(tcp->ssl) > 0)
      return 0;
 #endif
-   
+
    while (1)
      {
 	int ret;
-	
+
 	FD_ZERO(&fds);
 	FD_SET(fd, &fds);
 	tv.tv_sec = SLtcp_TimeOut_Secs;
 	tv.tv_usec = 0;
 
 	ret = select(fd + 1, &fds, NULL, NULL, &tv);
-	
+
 	if (ret > 0)
 	  return 0;
-	
+
 	if (ret == 0)
 	  return -1;		       /* timed out */
-	
+
 	if (errno == EINTR)
 	  {
 	     if (-1 == sys_call_interrupted_hook ())
 	       return -1;
-	     
+
 	     continue;
 	  }
-	
+
 	return -1;
      }
 }
@@ -899,32 +944,32 @@ static int do_ssl_read (SSL *ssl, char *buf, unsigned int len)
 	     if (-1 == sys_call_interrupted_hook ())
 	       return -1;
 	     break;
-	     
+
 	   case SSL_ERROR_SYSCALL:
 	     if (-1 == check_errno ())
 	       return -1;
 	     break;
 	  }
-	
+
      }
-   
+
    return nread;
 }
 #endif
 static int do_read (SLTCP_Type *tcp) /*{{{*/
-{   
+{
    int nread, fd;
-   
+
    if (tcp->tcp_flags & SLTCP_EOF_FLAG)
      return -1;
 
    fd = tcp->tcp_fd;
-   
+
 #if !defined(VMS) && !defined(USE_WINSOCK_SLTCP)
    /* The wait_for_input function call is probably not necessary.  The reason
     * that I am using it is that select will return EINTR if interrupted by
-    * a signal and most implementations will not restart it.  In any case, 
-    * slrn attempts to set signals to NOT restart system calls.  In such a 
+    * a signal and most implementations will not restart it.  In any case,
+    * slrn attempts to set signals to NOT restart system calls.  In such a
     * case, the read below can be interrupted.
     */
    if (-1 == wait_for_input (tcp))
@@ -934,7 +979,7 @@ static int do_read (SLTCP_Type *tcp) /*{{{*/
 #if SLTCP_HAS_SSL_SUPPORT
    if (tcp->ssl != NULL)
      nread = do_ssl_read (tcp->ssl, (char *)tcp->tcp_read_ptr, SLTCP_BUF_SIZE);
-   else 
+   else
 #endif
      while (-1 == (nread = SLTCP_READ (fd, (char *)tcp->tcp_read_ptr, SLTCP_BUF_SIZE)))
        {
@@ -947,7 +992,7 @@ static int do_read (SLTCP_Type *tcp) /*{{{*/
 
    tcp->tcp_read_ptr_max += (unsigned int) nread;
    tcp->bytes_in += (unsigned int) nread;
-   
+
    return nread;
 }
 
@@ -959,17 +1004,17 @@ unsigned int sltcp_read (SLTCP_Type *tcp, char *s, unsigned int len) /*{{{*/
    unsigned char *buf, *b;
    unsigned int blen;
    unsigned int total_len;
-   
+
    if ((tcp == NULL) || (-1 == (fd = tcp->tcp_fd)))
      return 0;
-   
+
    total_len = 0;
-   
+
    while (1)
      {
 	buf = tcp->tcp_read_ptr;
 	b = tcp->tcp_read_ptr_max;
-   
+
 	blen = (unsigned int) (b - buf);
 	if (blen >= len)
 	  {
@@ -978,7 +1023,7 @@ unsigned int sltcp_read (SLTCP_Type *tcp, char *s, unsigned int len) /*{{{*/
 	     total_len += len;
 	     return total_len;
 	  }
-   
+
 	if (blen)
 	  {
 	     memcpy (s, (char *) buf, blen);
@@ -986,9 +1031,9 @@ unsigned int sltcp_read (SLTCP_Type *tcp, char *s, unsigned int len) /*{{{*/
 	     s += blen;
 	     len -= blen;
 	  }
-	
+
 	tcp->tcp_read_ptr = tcp->tcp_read_ptr_max = tcp->tcp_read_buf;
-	
+
 	if (-1 == do_read (tcp))
 	  return total_len;
      }
@@ -1001,52 +1046,51 @@ unsigned int sltcp_write (SLTCP_Type *tcp, char *s, unsigned int len) /*{{{*/
    int fd;
    unsigned int blen;
    unsigned char *b;
-   
+
    if ((tcp == NULL) || (-1 == (fd = tcp->tcp_fd)))
      return 0;
-   
+
    b = tcp->tcp_write_ptr;
    blen = (unsigned int) ((tcp->tcp_write_buf + SLTCP_BUF_SIZE) - b);
    if (len <= blen)
      {
 	memcpy ((char *) b, s, len);
 	tcp->tcp_write_ptr += len;
-	
+
 	return len;
      }
-   
+
    if (-1 == sltcp_flush_output (tcp))
      return 0;
-   
+
    return do_write (tcp, (unsigned char *) s, len);
 }
 
 /*}}}*/
-   
+
 int sltcp_map_service_to_port (char *service) /*{{{*/
 {
    struct servent *sv;
-   
+
    sv = getservbyname (service, "tcp");
    if (sv == NULL) return -1;
-   
+
    return (int) ntohs (sv->s_port);
 }
 
 /*}}}*/
 
-
 int sltcp_fputs (SLTCP_Type *tcp, char *s) /*{{{*/
 {
    unsigned int len;
-   
+
    if (s == NULL)
      return -1;
-   
+
    len = strlen (s);
    if (len != sltcp_write (tcp, s, len))
      return -1;
-   
+
    return 0;
 }
 
@@ -1055,12 +1099,12 @@ int sltcp_fputs (SLTCP_Type *tcp, char *s) /*{{{*/
 int sltcp_vfprintf (SLTCP_Type *tcp, char *fmt, va_list ap) /*{{{*/
 {
    char buf [SLTCP_BUF_SIZE];
-   
+
    if ((tcp == NULL) || (-1 == tcp->tcp_fd))
      return -1;
-   
-   slrn_vsnprintf (buf, sizeof (buf), fmt, ap);
-   
+
+   (void) SLvsnprintf (buf, sizeof (buf), fmt, ap);
+
    return sltcp_fputs (tcp, buf);
 }
 
@@ -1071,31 +1115,31 @@ int sltcp_fgets (SLTCP_Type *tcp, char *buf, unsigned int len) /*{{{*/
    unsigned char *r, *rmax;
    char *buf_max;
    int fd;
-   
+
    if ((tcp == NULL) || (-1 == (fd = tcp->tcp_fd)) || (len == 0))
      return -1;
-   
+
    buf_max = buf + (len - 1);	       /* allow room for \0 */
-   
+
    while (1)
      {
 	r = tcp->tcp_read_ptr;
 	rmax = tcp->tcp_read_ptr_max;
-   
+
 	while (r < rmax)
 	  {
 	     unsigned char ch;
-	     
+
 	     if (buf == buf_max)
 	       {
 		  *buf = 0;
 		  tcp->tcp_read_ptr = r;
 		  return 0;
 	       }
-	     
+
 	     ch = *r++;
 	     *buf++ = ch;
-	     
+
 	     if (ch == '\n')
 	       {
 		  *buf = 0;
@@ -1103,9 +1147,9 @@ int sltcp_fgets (SLTCP_Type *tcp, char *buf, unsigned int len) /*{{{*/
 		  return 0;
 	       }
 	  }
-	
+
 	tcp->tcp_read_ptr_max = tcp->tcp_read_ptr = tcp->tcp_read_buf;
-	
+
 	if (-1 == do_read (tcp))
 	  return -1;
      }
@@ -1151,7 +1195,7 @@ int sltcp_close_sltcp (void)
    WSACleanup ();
    Winsock_Started = 0;
 #endif
-   
+
    return 0;
 }
 
@@ -1184,14 +1228,12 @@ int sltcp_get_fd (SLTCP_Type *tcp)
    return tcp->tcp_fd;
 }
 
-
-
 #if 0 && defined(USE_WINSOCK_SLTCP)
 /* Some winsock notes */
 
 /* To interrupt a read/write on a socket, it may be useful to set a handler
  * control C, e.g., define something like
- * 
+ *
  *    SetConsoleCtrlHandler (kbd_int_handler, TRUE);
  *
  * where:
