@@ -1,7 +1,7 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 1994, 1999, 2007-2012 John E. Davis <jed@jedsoft.org>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -14,8 +14,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #ifndef _SLTCP_H_LOADED_
 #define _SLTCP_H_LOADED_
@@ -39,6 +39,7 @@ extern int sltcp_fgets (SLTCP_Type *, char *, unsigned int);
 
 extern int sltcp_open_sltcp (void);
 extern int sltcp_close_sltcp (void);
+extern void (*SLtcp_Verror_Hook) (char *, va_list);
 
 extern unsigned int sltcp_get_num_input_bytes (SLTCP_Type *);
 extern unsigned int sltcp_get_num_output_bytes (SLTCP_Type *);
