@@ -1,8 +1,8 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2002 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 1994, 1999, 2007-2012 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2002-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -49,146 +49,56 @@
 #include "util.h"
 #include "ttymsg.h"
 #include "snprintf.h"
+#include "slrn.h"
+#include "strutil.h"
+#include "common.h"
 
-/* This function allows NULL as a parameter. This fact _is_ exploited */
-char *slrn_skip_whitespace (char *b) /*{{{*/
+size_t slrn_charset_strlen (const char *str, char *cset)
 {
-   if (b == NULL) return NULL;
-   
-   while (isspace (*b))
-     b++;
-
-   return b;
+  if ((cset != NULL) && !slrn_case_strcmp(cset,"utf-8"))
+     return SLutf8_strlen ((SLuchar_Type *)str, 1);
+  else
+     return strlen (str);
 }
 
-/*}}}*/
-
-char *slrn_bskip_whitespace (char *smin)
+/* Find out how many characters (columns) a string would use on screen.
+ * If len>=0, only the first len bytes are examined.
+ */
+int slrn_screen_strlen (const char *s, const char *smax) /*{{{*/
 {
-   char *s;
+   if (smax == NULL)
+     smax = s + strlen (s);
 
-   if (smin == NULL) return NULL;
-   s = smin + strlen (smin);
-
-   while (s > smin)
+#if SLANG_VERSION >= 20000
+   if (Slrn_UTF8_Mode)
      {
-	s--;
-	if (0 == isspace(*s))
-	  return s + 1;
+	return SLsmg_strwidth ((SLuchar_Type *) s, (SLuchar_Type *) smax);
      }
-   return s;
-}
-
-   
-/* returns a pointer to the end of the string */
-char *slrn_trim_string (char *s) /*{{{*/
-{
-   s = slrn_bskip_whitespace (s);
-   if (s != NULL)
-     *s = 0;
-
-   return s;
-}
-/*}}}*/
-
-char *slrn_strchr (char *s, char ch) /*{{{*/
-{
-   register char ch1;
-   
-   while (((ch1 = *s) != 0) && (ch != ch1)) s++;
-   if (ch1 == 0) return NULL;
-   return s;
-}
-
-/*}}}*/
-
-/* Search for characters from list in string str.  If found, return a pointer
- * to the first occurrence.  If not found, return NULL. */
-char *slrn_strbrk (char *str, char *list) /*{{{*/
-{
-   char ch, ch1, *p;
-   
-   while ((ch = *str) != 0)
+   else
+#endif
      {
-	p = list;
-	while ((ch1 = *p) != 0)
+	int retval = 0;
+	while (s < smax)
 	  {
-	     if (ch == ch1) return str;
-	     p++;
+	     unsigned char ch= (unsigned char) *s++;
+
+	     if ((ch == '\t') && (SLsmg_Tab_Width > 0))
+	       {
+		  retval += SLsmg_Tab_Width;
+		  retval -= retval % SLsmg_Tab_Width;
+	       }
+	     else if (((ch >= ' ') && (ch < 127))
+		      || (ch >= (unsigned char) SLsmg_Display_Eight_Bit))
+	       retval++;
+	     else
+	       {
+		  retval += 2;	       /* ^X */
+		  if (ch & 0x80) retval += 2;   /* <XX> */
+	       }
 	  }
-	str++;
+	return retval;
      }
-   return NULL;
 }
-
-/*}}}*/
-
-char *slrn_simple_strtok (char *s, char *chp) /*{{{*/
-{
-   static char *s1;
-   char ch = *chp;
-   
-   if (s == NULL)
-     {
-	if (s1 == NULL) return NULL;
-	s = s1;
-     }
-   else s1 = s;
-   
-   while (*s1 && (*s1 != ch)) s1++;
-   
-   if (*s1 == 0)
-     {
-	s1 = NULL;
-     }
-   else *s1++ = 0;
-   return s;
-}
-
-/*}}}*/
-
-
-int slrn_case_strncmp (unsigned char *a, register unsigned char *b, register unsigned int n) /*{{{*/
-{
-   register unsigned char cha, chb, *bmax;
-   
-   bmax = b + n;
-   while (b < bmax)
-     {
-	cha = UPPER_CASE(*a);
-	chb = UPPER_CASE(*b);
-	if (cha != chb)
-	  {
-	     return (int) cha - (int) chb;
-	  }
-	else if (chb == 0) return 0;
-	b++;
-	a++;
-     }
-   return 0;
-}
-
-/*}}}*/
-
-int slrn_case_strcmp (unsigned char *a, register unsigned char *b) /*{{{*/
-{
-   register unsigned char cha, chb;
-   
-   while (1)
-     {
-	cha = UPPER_CASE(*a);
-	chb = UPPER_CASE(*b);
-	if (cha != chb)
-	  {
-	     return (int) cha - (int) chb;
-	  }
-	else if (chb == 0) break;
-	b++;
-	a++;
-     }
-   return 0;
-}
-
 /*}}}*/
 
 #if defined(IBMPC_SYSTEM)
@@ -213,24 +123,24 @@ int slrn_cygwin_convert_path (char *inpath, char *outpath, size_t n)
 {
    unsigned int outlen;
    char *p;
-   
+
    /*
     **  first, a quick sanity check to look for invalid formats
     */
    p = strrchr (inpath, ':');        /* we'll re-use 'p' later on */
    if (p && (p != inpath+1))
      return -1;
-   
+
    /*
     **  let's do some size checking
     */
    outlen = strlen (inpath) + 1;
    if (p)
      outlen += 9;        /* "/cygdrive/c" (11) replaces "c:" (2) ==> 9 */
-   
+
    if (n < outlen)
      return -2;
-   
+
    /*
     **  paths starting with C:, (or D:, etc) should be converted to the
     **  Cygwin native format /cygdrive/c (or /cygdrive/d, etc) while copying
@@ -256,7 +166,7 @@ int slrn_cygwin_convert_path (char *inpath, char *outpath, size_t n)
 	  *p = '/';
         p++;
      }
-   
+
    /*
     **  normal return
     */
@@ -292,7 +202,7 @@ static void fixup_path (char *path) /*{{{*/
 {
 #ifndef VMS
    unsigned int len;
-   
+
    len = strlen (path);
    if (len == 0) return;
 # ifdef IBMPC_SYSTEM
@@ -306,7 +216,7 @@ static void fixup_path (char *path) /*{{{*/
 
 /*}}}*/
 
-/* dir and file could be the same in which case this performs a strcat. 
+/* dir and file could be the same in which case this performs a strcat.
  * If name looks like an absolute path, it will be returned.
  */
 int slrn_dircat (char *dir, char *name, char *file, size_t n)
@@ -315,8 +225,8 @@ int slrn_dircat (char *dir, char *name, char *file, size_t n)
 #ifdef __CYGWIN__
    char convdir [SLRN_MAX_PATH_LEN];
 #endif
-   
-   if (name != NULL) 
+
+   if (name != NULL)
      {
 	if (slrn_is_absolute_path (name))
 	  {
@@ -329,20 +239,20 @@ int slrn_dircat (char *dir, char *name, char *file, size_t n)
 #endif
 	     return 0;
 	  }
-	
+
 	len = strlen (name);
      }
-   
+
    if (dir != NULL) len += strlen (dir);
-   
+
    len += 2;			       /* for / and \0 */
    if (len > n)
      {
 	slrn_error (_("File name too long."));
 	return -1;
      }
-   
-   if (dir != NULL) 
+
+   if (dir != NULL)
      {
 	if (dir != file) strcpy (file, dir); /* safe */
 	fixup_path (file);
@@ -364,83 +274,6 @@ int slrn_dircat (char *dir, char *name, char *file, size_t n)
    return 0;
 }
 
-/*{{{ Memory Allocation Routines */
-
-static char *do_malloc_error (int do_error)
-{
-   if (do_error) slrn_error (_("Memory allocation failure."));
-   return NULL;
-}
-
-char *slrn_safe_strmalloc (char *s) /*{{{*/
-{
-   s = SLmake_string (s);
-   if (s == NULL) slrn_exit_error (_("Out of memory."));
-   return s;
-}
-
-/*}}}*/
-
-char *slrn_strnmalloc (char *s, unsigned int len, int do_error)
-{
-   s = SLmake_nstring (s, len);
-   
-   if (s == NULL)
-     return do_malloc_error (do_error);
-   
-   return s;
-}
-
-char *slrn_strmalloc (char *s, int do_error)
-{
-   if (s == NULL) return NULL;
-   return slrn_strnmalloc (s, strlen (s), do_error);
-}
-
-
-char *slrn_malloc (unsigned int len, int do_memset, int do_error)
-{   
-   char *s;
-   
-   s = (char *) SLmalloc (len);
-   if (s == NULL)
-     return do_malloc_error (do_error);
-
-   if (do_memset)
-     memset (s, 0, len);
-   
-   return s;
-}
-
-char *slrn_realloc (char *s, unsigned int len, int do_error)
-{
-   if (s == NULL)
-     return slrn_malloc (len, 0, do_error);
-   
-   s = SLrealloc (s, len);
-   if (s == NULL)
-     return do_malloc_error (do_error);
-	
-   return s;
-}
-
-char *slrn_safe_malloc (unsigned int len)
-{
-   char *s;
-   
-   s = slrn_malloc (len, 1, 0);
-
-   if (s == NULL)
-     slrn_exit_error (_("Out of memory"));
-   
-   return s;
-}
-
-void slrn_free (char *s)
-{
-   if (s != NULL) SLfree (s);
-}
-
 void slrn_free_argc_argv_list (unsigned int argc, char **argv)
 {
    while (argc)
@@ -450,8 +283,6 @@ void slrn_free_argc_argv_list (unsigned int argc, char **argv)
      }
 }
 
-/*}}}*/
-
 char *slrn_fix_regexp (char *pat) /*{{{*/
 {
    static char newpat[256];
@@ -471,7 +302,7 @@ char *slrn_fix_regexp (char *pat) /*{{{*/
 
    if (len > sizeof(newpat))
      slrn_exit_error (_("Pattern too long for buffer"));
-     
+
    p = newpat;
 
    *p++ = '^';
@@ -484,8 +315,8 @@ char *slrn_fix_regexp (char *pat) /*{{{*/
 
 	*p++ = ch;
      }
-   
-   if (*(p - 1) != '$') 
+
+   if (*(p - 1) != '$')
      *p++ = '$';
 
    *p = 0;
@@ -511,7 +342,6 @@ int slrn_is_absolute_path (char *path)
    return 0;
 }
 
-
 /* This is like slrn_dircat except that any dots in name can get mapped to
  * slashes.  It also mallocs space for the resulting file.
  */
@@ -582,18 +412,17 @@ int slrn_fclose (FILE *fp) /*{{{*/
 
 /*}}}*/
 
-
 int slrn_file_exists (char *file) /*{{{*/
 {
    struct stat st;
    int m;
-   
+
 #ifdef _S_IFDIR
 # ifndef S_IFDIR
 #  define S_IFDIR _S_IFDIR
 # endif
 #endif
-   
+
 #ifndef S_ISDIR
 # ifdef S_IFDIR
 #  define S_ISDIR(m) (((m) & S_IFMT) == S_IFDIR)
@@ -601,13 +430,13 @@ int slrn_file_exists (char *file) /*{{{*/
 #  define S_ISDIR(m) 0
 # endif
 #endif
-   
+
    if (file == NULL)
      return -1;
 
    if (stat(file, &st) < 0) return 0;
    m = st.st_mode;
-   
+
    if (S_ISDIR(m)) return (2);
    return 1;
 }
@@ -618,13 +447,13 @@ int slrn_file_size (char *file) /*{{{*/
 {
    struct stat st;
    int m;
-   
+
    if (file == NULL)
      return 0;
-   
+
    if (stat(file, &st) < 0) return -1;
    m = st.st_mode;
-   
+
    if (S_ISDIR(m)) return -1;
    return (int)st.st_size;
 }
@@ -634,14 +463,14 @@ char *slrn_basename (char *file)
 {
    char *f;
 #ifdef VMS
-   f = slrn_strchr (file, ']');
+   f = slrn_strbyte (file, ']');
    if (f != NULL) return f + 1;
    return file;
 #else
 
-   while (NULL != (f = slrn_strchr (file, SLRN_PATH_SLASH_CHAR)))
+   while (NULL != (f = slrn_strbyte (file, SLRN_PATH_SLASH_CHAR)))
      file = f + 1;
-   
+
    return file;
 #endif
 }
@@ -662,10 +491,10 @@ static int file_eqs (char *a, char *b)
 #ifdef REAL_UNIX_SYSTEM
    struct stat st_a, st_b;
 #endif
-   
+
    if (0 == strcmp (a, b))
      return 1;
-   
+
 #ifndef REAL_UNIX_SYSTEM
    return 0;
 #else
@@ -673,20 +502,18 @@ static int file_eqs (char *a, char *b)
      return 0;
    if (-1 == stat (b, &st_b))
      return 0;
-   
+
    return ((st_a.st_ino == st_b.st_ino)
 	   && (st_a.st_dev == st_b.st_dev));
 #endif
 }
 
-   
-
 int slrn_copy_file (char *infile, char *outfile)
 {
    FILE *in, *out;
    int ch;
    int ret;
-   
+
    if ((infile == NULL) || (outfile == NULL))
      return -1;
 
@@ -698,7 +525,7 @@ int slrn_copy_file (char *infile, char *outfile)
 	slrn_error (_("Error opening %s"), infile);
 	return -1;
      }
-   
+
    if (NULL == (out = fopen (outfile, "wb")))
      {
 	fclose (in);
@@ -716,11 +543,11 @@ int slrn_copy_file (char *infile, char *outfile)
 	     break;
 	  }
      }
-   
+
    fclose (in);
    if (-1 == slrn_fclose (out))
      ret = -1;
-   
+
    return ret;
 }
 
@@ -742,6 +569,107 @@ int slrn_move_file (char *infile, char *outfile)
    return 0;
 }
 
+/* Some functions to handle file backups correctly ... */
+
+/* Make and return a malloc'ed filename by adding an OS-specific suffix
+ * that flags backup files. */
+char *slrn_make_backup_filename (char *filename)
+{
+#ifdef SLRN_USE_OS2_FAT
+   unsigned int len = strlen(filename)+5;
+   char *retval = slrn_safe_malloc (len);
+   slrn_os2_make_fat (retval, len, filename, ".bak");
+   return retval;
+#else
+   unsigned int len;
+   char *suffix;
+   char *retval;
+# ifdef VMS
+   suffix = "-bak";
+# else
+   suffix = "~";
+# endif
+   len = 1 + strlen (suffix) + strlen (filename);
+   retval = slrn_safe_malloc (len);
+   (void) sprintf (retval, "%s%s", filename, suffix);
+   return retval;
+#endif
+}
+
+/* Creates a backup of the given file, copying it if necessary (i.e. it is
+ * a softlink or there are multiple hardlinks on it) - after that, filename
+ * may or may no longer denote an existing file.
+ * Returns 0 on success, -1 on errors. */
+int slrn_create_backup (char *filename)
+{
+   char *backup_file = slrn_make_backup_filename (filename);
+   int retval = -1, do_copy = 0;
+#ifdef __unix__
+   struct stat st;
+
+   if (0 == lstat (filename, &st))
+     {
+	do_copy = (st.st_nlink > 1)
+# ifdef S_ISLNK
+	  || (S_ISLNK(st.st_mode))
+# endif
+	    ;
+     }
+#endif /* __unix__ */
+
+   if (slrn_file_exists(filename))
+     {
+	if (!do_copy)
+	  retval = rename (filename, backup_file);
+	if (retval == -1)
+	  retval = slrn_copy_file (filename, backup_file);
+     }
+
+   SLfree (backup_file);
+
+   return retval;
+}
+
+/* Tries to delete the backup of the given file, ignoring all errors. */
+void slrn_delete_backup (char *filename)
+{
+   char *backup_file = slrn_make_backup_filename (filename);
+   (void) slrn_delete_file (backup_file);
+   SLfree (backup_file);
+}
+
+/* Tries to restore the given file from the backup, copying it if necessary
+ * (same condition as above). */
+int slrn_restore_backup (char *filename)
+{
+   char *backup_file = slrn_make_backup_filename (filename);
+   int retval = -1, do_copy = 0;
+#ifdef __unix__
+   struct stat st;
+
+   if (0 == lstat (filename, &st))
+     {
+	do_copy = (st.st_nlink > 1)
+# ifdef S_ISLNK
+	  || (S_ISLNK(st.st_mode))
+# endif
+	    ;
+     }
+#endif /* __unix__ */
+
+   if (!do_copy)
+     retval = rename (backup_file, filename);
+   if (retval == -1)
+     {
+	retval = slrn_copy_file (backup_file, filename);
+	if (retval == 0)
+	  (void) slrn_delete_file (backup_file);
+     }
+
+   SLfree (backup_file);
+
+   return retval;
+}
 
 unsigned int slrn_sleep (unsigned int len)
 {
